# Load packages and read in data
library(ggplot2)
library(reshape2)
library(nlme)
library(rstan)
options(mc.cores = parallel::detectCores())
dat <- read.csv("attsnarc.csv", header=TRUE)

# Prepare data for Stan
dat$SubjectID <- as.numeric(factor(dat$SubjectID, levels=unique(dat$SubjectID)))
dat$LabID <- as.numeric(factor(dat$LabID, levels=unique(dat$LabID)))
colnames(dat)[4] <- "Effect"
dat$EffectID <- as.numeric(as.factor(dat$Effect))
dat$ConditionID <- as.numeric(as.factor( paste(dat$Location,dat$Magnitude,sep="*") ))

# Prepare scalars for stan and study / condition labels
nCond <- max(dat$ConditionID)
nLab <- max(dat$LabID)
nSubj <- max(dat$SubjectID)
nRow <- nrow(dat)
nDepvar <- length(unique(dat$EffectID))

study <- colnames(table(dat$EffectID, dat$Effect))
study <- paste(study, "ms\nDelay")
cond <- colnames(table(dat$ConditionID, paste(dat$Location,dat$Magnitude,sep="*")))
study.cond <- paste( rep(study,each=nCond), rep(cond,nDepvar), sep="*" )
cond.study <- paste( rep(study,nCond), rep(cond,each=nDepvar), sep="*" )
ulab <- sort(unique(dat$LabID))

# Converting to centiseconds so parameters are on a roughly unit scale to improve convergence
dat$value <- 100 * dat$value

# Prepare initial values
m1 <- lme(value ~ factor(ConditionID)*factor(EffectID) - factor(ConditionID)-factor(EffectID)-1,
		random=list(~1|LabID, ~1|SubjectID),
		weights=varIdent(form=~1|EffectID),
		data=dat)
init.fcn <- function(){
	alpha_cd <- matrix(fixef(m1), nCond, nDepvar)
	sigma_d <- c(1, as.vector(exp(m1$modelStruct$varStruct))) * m1$sigma
	
	phi_beta_lf <- matrix(0, nLab, 3)
	phi_beta_lf[,1] <- ranef(m1)$LabID[,1]
	omega_phi_beta_f <- c(as.numeric(VarCorr(m1))[7], 1, 1)
	Rho_phi_beta_ff <- diag(3)
	lambda_beta1_d <- rep(0, nDepvar)
	lambda_beta2_cd <- matrix(0, nCond, nDepvar)
	
	eta_ldc <- array(0, c(nLab, nDepvar, nCond))
	omega_eta_dc <- matrix(1, nDepvar, nCond)
	Rho_eta_dcc <- array(0, c(nDepvar, nCond, nCond))
	for(i in 1:nDepvar){ Rho_eta_dcc[i,,] <- diag(nCond) }
		
	phi_gamma_sf <- matrix(0, nSubj, 3)
	phi_gamma_sf[,1] <- ranef(m1)$SubjectID[,1]
	omega_phi_gamma_f <- c(as.numeric(VarCorr(m1))[9], 1, 1)
	Rho_phi_gamma_ff <- diag(3)
	lambda_gamma1_d <- rep(0, nDepvar)
	lambda_gamma2_cd <- matrix(0, nCond, nDepvar)
	
	return(list( alpha_cd=alpha_cd, sigma_d=sigma_d,
		phi_beta_lf=phi_beta_lf, omega_phi_beta_f=omega_phi_beta_f, Rho_phi_beta_ff=Rho_phi_beta_ff,
		lambda_beta1_d=lambda_beta1_d, lambda_beta2_cd=lambda_beta2_cd,
		eta_ldc=eta_ldc, omega_eta_dc=omega_eta_dc, Rho_eta_dcc=Rho_eta_dcc,
		phi_gamma_sf=phi_gamma_sf, omega_phi_gamma_f=omega_phi_gamma_f, Rho_phi_gamma_ff=Rho_phi_gamma_ff,
		lambda_gamma1_d=lambda_gamma1_d, lambda_gamma2_cd=lambda_gamma2_cd	))
}

# Compile model and run sampler
stan.model <- stan_model(file="model1.stan")
post <- sampling(stan.model, seed=13579, verbose=TRUE, refresh=10,
				  pars=c("mu","sig"), include=FALSE,
				  data=list(
					  nCond=nCond, nLab=nLab, nSubj=nSubj,nRow=nRow, nDepvar=nDepvar,
					  y=dat$value, subj=dat$SubjectID, lab=dat$LabID,
					  depvar=dat$EffectID, condition=dat$ConditionID, dv_scale=init.fcn()$sigma_d
				  ), 
				  init=init.fcn
				 )

# Post-process chains
# Step One: Impose identifiability constraints
P.fcn <- function(theta){
	P <- diag(3)
	P[1:2,2] <- theta[1:2]
	P[1:3,3] <- theta[3:5]
	return(P)
}
vcv.fcn <- function(theta,LL,OO,Pf){
	P <- Pf(theta)
	Pi <- solve(P)
	LLp <- LL%*%P
	OOp <- Pi%*%OO%*%t(Pi)
	return(list(LL=LLp, OO=OOp, VV=LLp%*%OOp%*%t(LLp), P=P, Pi=Pi))
}
d.idx2 <- seq(1, nCond*nDepvar, by=nCond)
d.idx3 <- seq(1, nCond*nDepvar, by=1)
dist.fcn <- function(theta,LL,OO){
	tmp <- vcv.fcn(theta,LL,OO,P.fcn)
	LLp <- tmp$LL
	OOp <- tmp$OO
	RRp <- cov2cor(OOp)
	d <- (mean(LLp[d.idx2,2]) - 0)^2 + (sd(LLp[d.idx2,2]) - 1)^2 +
		 (mean(LLp[d.idx3,3]) - 0)^2 + (sd(LLp[d.idx3,3]) - 1)^2 +
		 RRp[2,3]^2
	return(d)
}

aa <- as.array(post)
n.iter.post <- dim(aa)[[1]]
n.chains <- dim(aa)[[2]]
aa2 <- aa
theta.id <- c(0,1,   0,0,1)

for(j in 1:n.chains){
	for(i in 1:n.iter.post){	
		tmpl2 <- aa[i,j,grepl("lambda_beta1_d\\[",dimnames(aa)[[3]])]
		tmpl3 <- aa[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa)[[3]])]
		tmpp <- rbind(aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")])
		tmpo <- aa[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa)[[3]])]
		tmpR <- matrix(aa[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa)[[3]])], 3, 3)
	
		LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
		OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
		VV <- LL %*% OO %*% t(LL)
		if(i==1){ tmp.init <- theta.id }else{ tmp.init <- oo$par }
		oo <- nlminb(tmp.init, dist.fcn, LL=LL, OO=OO)		
		tmp <- vcv.fcn(oo$par, LL, OO, P.fcn)
		tmppp <- solve(tmp$P) %*% tmpp
	
		aa2[i,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx2,2]
		aa2[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx3,3]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
		aa2[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])] <- sqrt(diag(tmp$OO))
		aa2[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa2)[[3]])] <- as.vector(tmp$OO)
		aa2[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa2)[[3]])] <- as.vector(cov2cor(tmp$OO))
	}
}

for(j in 1:n.chains){
	for(i in 1:n.iter.post){	
		tmpl2 <- aa[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa)[[3]])]
		tmpl3 <- aa[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa)[[3]])]
		tmpp <- rbind(aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")])
		tmpo <- aa[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa)[[3]])]
		tmpR <- matrix(aa[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa)[[3]])], 3, 3)
	
		LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
		OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
		VV <- LL %*% OO %*% t(LL)
		if(i==1){ tmp.init <- theta.id }else{ tmp.init <- oo$par }
		oo <- nlminb(tmp.init, dist.fcn, LL=LL, OO=OO)		
		tmp <- vcv.fcn(oo$par, LL, OO, P.fcn)
		tmppp <- solve(tmp$P) %*% tmpp
	
		aa2[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx2,2]
		aa2[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx3,3]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
		aa2[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])] <- sqrt(diag(tmp$OO))
		aa2[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa2)[[3]])] <- as.vector(tmp$OO)
		aa2[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa2)[[3]])] <- as.vector(cov2cor(tmp$OO))
	}
}

# Step Two: Resolve within chain reflection invariance
chain.relabel <- function(LL, fac.idx, col.idx){
	# Erosheva and Curtis, Psychometrika (2017)
	qq <- max(fac.idx)
	if(length(col.idx)!=qq){print("error")}

	nu <- matrix(NA, nrow(LL), qq)
	LLr <- LL
	for(i in 1:qq){
		LLs <- LL[,col.idx[[i]]]
		nu1 <- rep(1, nrow(LL))
		crit <- TRUE
		while(crit){
			mm <- apply(LLs * nu1, 2, mean)
			ss <- apply(LLs * nu1, 2, sd)
			ll.plus <- colSums( apply(LLs, 1, dnorm, mean=mm, sd=ss, log=TRUE) )
			ll.minus <- colSums( apply(-LLs, 1, dnorm, mean=mm, sd=ss, log=TRUE) )
			
			nu0 <- nu1
			nu1 <- -1 + 2*(ll.plus > ll.minus)
			crit <- any(nu0!=nu1)		
		}
		
		nu[,i] <- nu1
		LLr[,col.idx[[i]]] <- LLs * nu1		
	}
	
	mm <- apply(LLr, 2, mean)
	ss <- apply(LLr, 2, sd)
	ll <- sum(colSums( apply(LLr, 1, dnorm, mean=mm, sd=ss, log=TRUE) ))
	out <- list(loadings=LLr, signs=nu, ll=ll)
}

aa3 <- aa2

factor.index <- rep(1:2, c(nDepvar,nDepvar*nCond))
column.index <- lapply(1:2, function(x){which(factor.index==x)})
for(j in 1:n.chains){
	tmpl2 <- aa2[,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])]
	tmpl3 <- aa2[,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])]
	tmpo <- aa2[,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])]
	
	tmpl2 <- tmpl2 * tmpo[,2]
	tmpl3 <- tmpl3 * tmpo[,3]
	
	LL <- cbind(tmpl2, tmpl3)
	tmp.relabel <- chain.relabel(LL, factor.index, column.index)
	
	tmp.idx <- which(rowSums(tmp.relabel$signs == -1) > 0)
	for(i in tmp.idx){	
		tmpl2 <- aa2[i,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])]
		tmpl3 <- aa2[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])]
		tmpp <- rbind(aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")])
		tmpo <- aa2[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])]
		tmpR <- matrix(aa2[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa2)[[3]])], 3, 3)

		flip1 <- tmp.relabel$signs[i,1] == -1
		flip2 <- tmp.relabel$signs[i,2] == -1
	
		if(flip1 | flip2){
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			if(flip2){ theta[3:5] <- -theta[3:5] }
			tmp <- vcv.fcn(theta,LL,OO,P.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa3[i,j,grepl("lambda_beta1_d\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx2,2]
			aa3[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx3,3]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
			aa3[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa3)[[3]])] <- sqrt(diag(tmp$OO))
			aa3[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa3)[[3]])] <- as.vector(tmp$OO)
			aa3[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa3)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}	
	}
}

for(j in 1:n.chains){
	tmpl2 <- aa2[,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])]
	tmpl3 <- aa2[,j,grepl("lambda_gamma2_cd\\[",dimnames(aa2)[[3]])]
	tmpo <- aa2[,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])]
	
	tmpl2 <- tmpl2 * tmpo[,2]
	tmpl3 <- tmpl3 * tmpo[,3]
	
	LL <- cbind(tmpl2, tmpl3)
	tmp.relabel <- chain.relabel(LL, factor.index, column.index)
	
	tmp.idx <- which(rowSums(tmp.relabel$signs == -1) > 0)
	for(i in tmp.idx){	
		tmpl2 <- aa2[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])]
		tmpl3 <- aa2[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa2)[[3]])]
		tmpp <- rbind(aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")])
		tmpo <- aa2[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])]
		tmpR <- matrix(aa2[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa2)[[3]])], 3, 3)

		flip1 <- tmp.relabel$signs[i,1] == -1
		flip2 <- tmp.relabel$signs[i,2] == -1
	
		if(flip1 | flip2){
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			if(flip2){ theta[3:5] <- -theta[3:5] }
			tmp <- vcv.fcn(theta,LL,OO,P.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa3[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx2,2]
			aa3[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx3,3]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
			aa3[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa3)[[3]])] <- sqrt(diag(tmp$OO))
			aa3[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa3)[[3]])] <- as.vector(tmp$OO)
			aa3[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa3)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}	
	}
}

# Step Three: Resolve between chain reflection invariance
tmp	<- aa3[,,"lambda_beta1_d[1]"]
chain.idx.lb1 <- as.vector(which(colMeans(tmp < 0) < 0.5))
tmp	<- aa3[,,"lambda_beta2_cd[1,1]"]
chain.idx.lb2 <- as.vector(which(colMeans(tmp < 0) < 0.5))
tmp	<- aa3[,,"lambda_gamma1_d[1]"]
chain.idx.lg1 <- as.vector(which(colMeans(tmp < 0) < 0.5))
tmp	<- aa3[,,"lambda_gamma2_cd[1,1]"]
chain.idx.lg2 <- as.vector(which(colMeans(tmp < 0) < 0.5))

aa4 <- aa3

for(j in 1:n.chains){	
	flip1 <- j %in% chain.idx.lb1
	flip2 <- j %in% chain.idx.lb2
	
	if(flip1 | flip2){			
		for(i in 1:n.iter.post){
			tmpl2 <- aa3[i,j,grepl("lambda_beta1_d\\[",dimnames(aa3)[[3]])]
			tmpl3 <- aa3[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa3)[[3]])]
			tmpp <- rbind(aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")])
			tmpo <- aa3[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa3)[[3]])]
			tmpR <- matrix(aa3[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa3)[[3]])], 3, 3)
	
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			if(flip2){ theta[3:5] <- -theta[3:5] }
			tmp <- vcv.fcn(theta,LL,OO,P.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa4[i,j,grepl("lambda_beta1_d\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx2,2]
			aa4[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx3,3]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
			aa4[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa4)[[3]])] <- sqrt(diag(tmp$OO))
			aa4[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa4)[[3]])] <- as.vector(tmp$OO)
			aa4[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa4)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}
	}
}

for(j in 1:n.chains){	
	flip1 <- j %in% chain.idx.lg1
	flip2 <- j %in% chain.idx.lg2
	
	if(flip1 | flip2){			
		for(i in 1:n.iter.post){
			tmpl2 <- aa3[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa3)[[3]])]
			tmpl3 <- aa3[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa3)[[3]])]
			tmpp <- rbind(aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")])
			tmpo <- aa3[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa3)[[3]])]
			tmpR <- matrix(aa3[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa3)[[3]])], 3, 3)
	
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			if(flip2){ theta[3:5] <- -theta[3:5] }
			tmp <- vcv.fcn(theta,LL,OO,P.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa4[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx2,2]
			aa4[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx3,3]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
			aa4[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa4)[[3]])] <- sqrt(diag(tmp$OO))
			aa4[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa4)[[3]])] <- as.vector(tmp$OO)
			aa4[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa4)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}
	}
}

# Plot output 
cond.orig <- cond
cond <- c("Left, large", "Left, small", "Right, large", "Right, small")
cond.order <- cond[c(2,1,4,3)]

fun.alpha <- function(y,alpha=0.05){ 
	out <- quantile(y,probs=c(alpha/2,0.5,1-alpha/2))
	names(out) <- c("ymin","y","ymax")
	out
}
f95 <- function(y){ fun.alpha(y,0.05) }
f50 <- function(y){ fun.alpha(y,0.50) }
ppp <- c(0.025, 0.25,   0.50,   0.75, 0.975)
qqq <- qnorm(ppp)

cc <- matrix(0, nDepvar, nCond*nDepvar)
for(i in 1:nDepvar){
	tmpj <- (i-1)*nCond + 1
	cc[i,tmpj:(tmpj+nCond-1)] <- c(-1,1,1,-1)
}
modk <- function(x,k){ tmp <- x%%k; ifelse(tmp==0,k,tmp) }

tmp1 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- apply(aa4[,,grepl("lambda_beta1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,rep(1:nDepvar, each=nCond)]
tmp3 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmpS <- apply(aa4[,,grepl("omega_phi_beta_f", dimnames(aa4)[[3]])], 3, identity)
for(i in 1:ncol(tmp1)){ tmp1[,i] <- 1 * tmpS[,1] }
for(i in 1:ncol(tmp2)){ tmp2[,i] <- tmp2[,i] * tmpS[,2] }
for(i in 1:ncol(tmp3)){ tmp3[,i] <- tmp3[,i] * tmpS[,3] }

tmpgg <- rbind( melt(tmp1),melt(tmp2),melt(tmp3) )
tmp.name <- c("Intercept", "Dependent Measure", "Variate")
tmpgg$Factor <- rep(tmp.name, c(length(tmp1),length(tmp2),length(tmp3)))
tmpgg$Factor <- factor(tmpgg$Factor, levels=unique(tmp.name))
tmpgg$Effect <- c( rep(study,each=nCond*nrow(tmp1)), rep(study,each=nCond*nrow(tmp2)), rep(study,each=nCond*nrow(tmp3)) )
tmpgg$Effect <- factor(tmpgg$Effect, levels=rev(study))
tmpc <- rep(cond, each=nrow(tmp3))
tmpgg$Condition <- c(rep(tmpc,nDepvar), rep(tmpc,nDepvar), rep(tmpc,nDepvar) )
tmpgg$Condition <- factor(tmpgg$Condition, levels=cond.order)
tmp1 <- data.frame(Factor=factor(tmp.name,levels=tmp.name), yy=c(NA,0,0))
tmp2 <- data.frame(Factor=factor(rep(tmp.name,each=2), levels=tmp.name),
						Effect=tmpgg$Effect[1], Condition=tmpgg$Condition[1],
						value=c(5,30,   -6,6,    -6,6)/10)
plot1 <- ggplot(tmpgg, aes(x=Effect,y=value*10,col=Condition)) + coord_flip() + 
		facet_grid(~Factor, scales="free_x") +
		stat_summary(fun.y=median, geom="point", position=position_dodge(width=-2/3)) + 
		stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-2/3), show.legend=FALSE) +
		stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-2/3)) +
		xlab("Dependent Measure") + ylab("Loading (ms)") + 
		geom_hline(data=tmp1,aes(yintercept=yy),linetype=2,size=0.25) +
		geom_point(data=tmp2,col=NA) +
		theme(legend.position="bottom")

ggsave("graph_paper_rrr1.pdf", plot=plot1, device=cairo_pdf, width=4/3*4.45, height=4/3*2.35)

tmp1 <- apply(aa4[,,grepl("lambda_gamma2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- apply(aa4[,,grepl("lambda_gamma1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,rep(1:nDepvar, each=nCond)]
tmp3 <- apply(aa4[,,grepl("lambda_gamma2_cd", dimnames(aa4)[[3]])], 3, identity)
tmpS <- apply(aa4[,,grepl("omega_phi_gamma_f", dimnames(aa4)[[3]])], 3, identity)
for(i in 1:ncol(tmp1)){ tmp1[,i] <- 1 * tmpS[,1] }
for(i in 1:ncol(tmp2)){ tmp2[,i] <- tmp2[,i] * tmpS[,2] }
for(i in 1:ncol(tmp3)){ tmp3[,i] <- tmp3[,i] * tmpS[,3] }

tmpgg <- rbind( melt(tmp1),melt(tmp2),melt(tmp3) )
tmp.name <- c("Intercept", "Dependent Measure", "Variate")
tmpgg$Factor <- rep(tmp.name, c(length(tmp1),length(tmp2),length(tmp3)) )
tmpgg$Factor <- factor(tmpgg$Factor, levels=unique(tmp.name))
tmpgg$Effect <- c( rep(study,each=nCond*nrow(tmp1)), rep(study,each=nCond*nrow(tmp2)), rep(study,each=nCond*nrow(tmp3)) )
tmpgg$Effect <- factor(tmpgg$Effect, levels=rev(study))
tmpc <- rep(cond, each=nrow(tmp3))
tmpgg$Condition <- c(rep(tmpc,nDepvar), rep(tmpc,nDepvar), rep(tmpc,nDepvar) )
tmpgg$Condition <- factor(tmpgg$Condition, levels=cond.order)
tmp1 <- data.frame(Factor=factor(tmp.name,levels=tmp.name), yy=c(NA,0,0))
tmp2 <- data.frame(Factor=factor(rep(tmp.name,each=2), levels=tmp.name),
						Effect=tmpgg$Effect[1], Condition=tmpgg$Condition[1],
						value=c(40,48,   -12,12,    -12,12)/10)
plot2 <- ggplot(tmpgg, aes(x=Effect,y=value*10,col=Condition)) + coord_flip() + 
		facet_grid(~Factor, scales="free_x") +
		stat_summary(fun.y=median, geom="point", position=position_dodge(width=-2/3)) + 
		stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-2/3), show.legend=FALSE) +
		stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-2/3)) +
		xlab("Dependent Measure") + ylab("Loading (ms)") + 
		geom_hline(data=tmp1,aes(yintercept=yy),linetype=2,size=0.25) +
		geom_point(data=tmp2,col=NA) +
		theme(legend.position="bottom")

ggsave("graph_paper_rrr2.pdf", plot=plot2, device=cairo_pdf, width=4/3*4.45, height=4/3*2.35)

tmp.idx0 <- 1:nDepvar
tmp.idx1 <- seq(1, nCond*nDepvar, by=nCond)
tmp.idx2 <- rep(tmp.idx0, each=nCond)
tmp.idx3 <- rep(tmp.idx0, each=nCond) + rep(c(0,nDepvar), nDepvar)
tmp2 <- apply(aa4[,,grepl("lambda_beta1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,tmp.idx2]
tmp3 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmpTc <- apply(aa4[,,grepl("Omega_phi_beta_ff", dimnames(aa4)[[3]])], 3, identity)
tmpTe <- apply(aa4[,,grepl("Omega_eta_dcc", dimnames(aa4)[[3]])], 3, identity)
tmpov <- matrix(NA, nrow(tmp2), ncol(tmp2))
tmpoc <- matrix(NA, nrow(tmp2), nrow(cc))
for(i in 1:nrow(tmp2)){		
	tmpa <- cbind(1, tmp2[i,], tmp3[i,])
	tmpb <- matrix(tmpTc[i,], 3, 3)
	tmpc <- tmpa %*% tmpb %*% t(tmpa)
	
	tmp.err <- matrix(0,nCond*nDepvar,nCond*nDepvar)
	tmp.Te <-  tmpTe[i,]
	for(j in 1:nDepvar){
		tmp <- matrix(tmp.Te[grepl(paste0("\\[",j), names(tmp.Te))], nDepvar, nDepvar)
		tmp.err[(1:nDepvar)+(j-1)*nDepvar,(1:nDepvar)+(j-1)*nDepvar] <- tmp
	}
	
	tmpd <- tmpc + tmp.err
	tmpov[i,] <- sqrt(diag( tmpd ))
	tmpoc[i,] <- sqrt(diag( cc %*% tmpd %*% t(cc) ))
}
tmpgg1v <- melt(tmpov)
tmpgg1v$Effect <- study[ceiling(tmpgg1v$Var2/nCond)]
tmpgg1v$Condition <- cond[modk(tmpgg1v$Var2,nCond)]
tmpgg1c <- melt(tmpoc)
tmpgg1c$Effect <- study[tmpgg1c$Var2]
tmpgg1c$Condition <- NA

tmp.idx0 <- 1:nDepvar
tmp.idx1 <- seq(1, nCond*nDepvar, by=nCond)
tmp.idx2 <- rep(tmp.idx0, each=nCond)
tmp.idx3 <- rep(tmp.idx0, each=nCond) + rep(c(0,nDepvar), nDepvar)
tmp2 <- apply(aa4[,,grepl("lambda_gamma1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,tmp.idx2]
tmp3 <- apply(aa4[,,grepl("lambda_gamma2_cd", dimnames(aa4)[[3]])], 3, identity)
tmpTc <- apply(aa4[,,grepl("Omega_phi_gamma_ff", dimnames(aa4)[[3]])], 3, identity)
tmpov <- matrix(NA, nrow(tmp2), ncol(tmp2))
tmpoc <- matrix(NA, nrow(tmp2), nrow(cc))
for(i in 1:nrow(tmp2)){		
	tmpa <- cbind(1, tmp2[i,], tmp3[i,])
	tmpb <- matrix(tmpTc[i,], 3, 3)
	tmpc <- tmpa %*% tmpb %*% t(tmpa)	
	tmpov[i,] <- sqrt(diag( tmpc ))
	tmpoc[i,] <- sqrt(diag( cc %*% tmpc %*% t(cc) ))
}
tmpgg2v <- melt(tmpov)
tmpgg2v$Effect <- study[ceiling(tmpgg2v$Var2/nCond)]
tmpgg2v$Condition <- cond[modk(tmpgg2v$Var2,nCond)]
tmpgg2c <- melt(tmpoc)
tmpgg2c$Effect <- study[tmpgg2c$Var2]
tmpgg2c$Condition <- NA

tmpgg <- rbind(tmpgg1v,tmpgg1c,tmpgg2v,tmpgg2c)
tmpgg$Effect <- factor(tmpgg$Effect, levels=rev(study))
tmpgg$Level <- rep(c("Lab","Subject"), c(nrow(tmpgg1v)+nrow(tmpgg1c),nrow(tmpgg2v)+nrow(tmpgg2c)))
tmpgg$facet <- rep(c("Variate","Effect","Variate","Effect"),
					c(nrow(tmpgg1v),nrow(tmpgg1c),nrow(tmpgg2v),nrow(tmpgg2c)))
tmpgg$facet <- factor(tmpgg$facet, levels=c("Variate", "Effect"))
tmp1 <- data.frame(Level="Lab", Effect=study[1], Condition="Left, large",
		facet=c("Variate", "Variate", "Effect", "Effect"),
		value=c(0,60,	0,6))
tmp1$facet <- factor(tmp1$facet, levels=c("Variate", "Effect"))
plot3 <- ggplot(tmpgg, aes(x=Effect,y=10*value, shape=Level, col=Condition)) + coord_flip() + 
	facet_grid(~facet, scales="free_x") +
	stat_summary(fun.y=median, geom="point", position=position_dodge(width=-2/3)) + 
	stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-2/3), show.legend=FALSE) +
	stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-2/3)) +
	xlab("Dependent Measure") + ylab("Variation (ms)") + 
	theme(legend.position="bottom", legend.box="vertical") + 
	scale_color_discrete("Condition", breaks=cond, na.value=1) + 
   	scale_shape_manual(values=c("Lab"=16, "Subject"=15)) +
	guides(shape = guide_legend(order = 1), color = guide_legend(nrow=2, order=2)) + 
	geom_point(data=tmp1, aes(y=value), col=NA)

ggsave("graph_paper_rrr3.pdf", plot=plot3, device=cairo_pdf, width=4/3*3.17, height=4/3*2.93)

ulab.id <- c("2t9re", "3s4uh", "5hgk4", "6ea38", "8gdp3", "ecdmw", "encwv", "fkjbz",
		"kwg95", "kx4w9", "nzxwr", "pqcf4", "t5y2h", "vzt3a", "xb6t2", "xmyd9", "yu8hp")
ulab.name <- c("University of Stirling, Scotland",
"University of Cambridge, England",
"University of Tübingen (Bryce), Germany",
"Czech Academy of Sciences, Czech Republic",
"University of Padua, Italy",
"University of Trento, Italy",
"University of Wisconsin-Madison, United States",
"University of Tübingen (Cipora), Germany",
"The University of Western Ontario, Canada",
"The University of Queensland, Australia",
"Leibniz-Institut für Wissensmedien, Germany",
"Colorado College, United States",
"Heinrich Heine University Düsseldorf, Germany",
"Tzu-Chi University, China",
"Erasmus University Rotterdam, Netherlands",
"University of Granada, Spain",
"University of Auckland, New Zealand")
names(ulab.name) <- ulab.id

tmp <- apply(aa4[,,grepl("beta_lcd", dimnames(aa4)[[3]])], 3, identity)
tmpd <- array(NA, c(nrow(tmp), nLab, 2))
for(i in 1:nrow(tmp)){
	tmpb <- matrix(tmp[i,], nrow=nLab)
	tmpd[i,,1] <- apply(tmpb, 1, function(x){sqrt(mean(x^2))})
	tmpc <- t(apply(tmpb, 1, function(x){cc%*%x}))
	tmpd[i,,2] <- apply(tmpc, 1, function(x){sqrt(mean(x^2))})
}
tmpgg <- melt(tmpd)
tmpgg$Lab <- ulab[tmpgg$Var2]
tmpgg$facet <- c("Variate", "Effect")[tmpgg$Var3]
tmpgg$facet <- factor(tmpgg$facet, levels=c("Variate", "Effect"))
tmp1 <- data.frame(LabName=ulab.name[1],
	facet=c("Variate", "Variate", "Effect", "Effect"),
	value=c(0,50,	0,6.5))
tmp1$facet <- factor(tmp1$facet, levels=c("Variate", "Effect"))

tmpgg$LabName <- ulab.name[tmpgg$Lab]
tmpo <- names(sort(by(tmpgg$value[tmpgg$facet=="Variate"], tmpgg$LabName[tmpgg$facet=="Variate"], median)))
tmpgg$LabName <- factor(tmpgg$LabName, levels=rev(tmpo))

plot5 <- ggplot(tmpgg, aes(x=LabName,y=10*value)) + coord_flip() + facet_wrap(~facet, scales="free_x") +
	stat_summary(fun.y=median, geom="point", position=position_dodge(width=-2/3)) + 
	stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-2/3), show.legend=FALSE) +
	stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-2/3)) +
	xlab("Lab") + ylab("Root Mean Square Error (ms)") +
	geom_point(data=tmp1, aes(y=value), col=NA)

ggsave("graph_paper_rrr5.pdf", plot=plot5, device=cairo_pdf, width=4/3*4.74, height=4/3*3.4)
