# Load packages and read in data
library(ggplot2)
library(reshape2)
library(nlme)
library(rstan)
options(mc.cores = parallel::detectCores())
dat <- read.csv("attsnarc.csv", header=TRUE)

# Prepare data for Stan
dat$SubjectID <- as.numeric(factor(dat$SubjectID, levels=unique(dat$SubjectID)))
dat$LabID <- as.numeric(factor(dat$LabID, levels=unique(dat$LabID)))
colnames(dat)[4] <- "Effect"
dat$EffectID <- as.numeric(as.factor(dat$Effect))
dat$ConditionID <- as.numeric(as.factor( paste(dat$Location,dat$Magnitude,sep="*") ))
dat$Match <- ifelse( (dat$Location=="left" & dat$Hand=="LH") | (dat$Location=="right" & dat$Hand=="RH"), 1, -1)
XX <- data.frame(x1 = dat$Match)

# Prepare scalars for stan and study / condition labels
nCond <- max(dat$ConditionID)
nCov <- ncol(XX)
nLab <- max(dat$LabID)
nSubj <- max(dat$SubjectID)
nRow <- nrow(dat)
nDepvar <- length(unique(dat$EffectID))

study <- colnames(table(dat$EffectID, dat$Effect))
study <- paste(study, "ms\nDelay")
cond <- colnames(table(dat$ConditionID, paste(dat$Location,dat$Magnitude,sep="*")))
study.cond <- paste( rep(study,each=nCond), rep(cond,nDepvar), sep="*" )
cond.study <- paste( rep(study,nCond), rep(cond,each=nDepvar), sep="*" )
ulab <- sort(unique(dat$LabID))

# Converting to centiseconds so parameters are on a roughly unit scale to improve convergence
dat$value <- 100 * dat$value

# Prepare initial values
m1 <- lme(value ~ factor(ConditionID)*factor(EffectID) + Match 
						- factor(ConditionID)-factor(EffectID)-1,
		random=list(~1|LabID, ~1+Match|SubjectID),
		weights=varIdent(form=~1|EffectID),
		data=dat)
init.fcn <- function(){
	alpha_int_cd <- matrix(fixef(m1)[-1], nCond, nDepvar)
	alpha_XX <- array(fixef(m1)[1], c(nCov))
	sigma_d <- c(1, as.vector(exp(m1$modelStruct$varStruct))) * m1$sigma
	
	phi_beta_lf <- matrix(0, nLab, 3)
	phi_beta_lf[,1] <- ranef(m1)$LabID[,1]
	omega_phi_beta_f <- c(as.numeric(VarCorr(m1))[8], 1, 1)
	Rho_phi_beta_ff <- diag(3)
	lambda_beta1_d <- rep(0, nDepvar)
	lambda_beta2_cd <- matrix(0, nCond, nDepvar)
	
	eta_ldc <- array(0, c(nLab, nDepvar, nCond))
	omega_eta_dc <- matrix(1, nDepvar, nCond)
	Rho_eta_dcc <- array(0, c(nDepvar, nCond, nCond))
	for(i in 1:nDepvar){ Rho_eta_dcc[i,,] <- diag(nCond) }
		
	phi_gamma_sf <- matrix(0, nSubj, 3)
	phi_gamma_sf[,1] <- ranef(m1)$SubjectID[,1]
	phi_gamma_sf[,3] <- ranef(m1)$SubjectID[,2]
	omega_phi_gamma_f <- c(as.numeric(VarCorr(m1))[10], 1, as.numeric(VarCorr(m1))[11])
	Rho_phi_gamma_ff <- diag(3)
	lambda_gamma1_d <- rep(0, nDepvar)
	
	return(list( alpha_int_cd=alpha_int_cd, alpha_XX=alpha_XX, sigma_d=sigma_d,
		phi_beta_lf=phi_beta_lf, omega_phi_beta_f=omega_phi_beta_f, Rho_phi_beta_ff=Rho_phi_beta_ff,
		lambda_beta1_d=lambda_beta1_d, lambda_beta2_cd=lambda_beta2_cd,
		eta_ldc=eta_ldc, omega_eta_dc=omega_eta_dc, Rho_eta_dcc=Rho_eta_dcc,
		phi_gamma_sf=phi_gamma_sf, omega_phi_gamma_f=omega_phi_gamma_f, Rho_phi_gamma_ff=Rho_phi_gamma_ff,
		lambda_gamma1_d=lambda_gamma1_d	))
}

# Compile model and run sampler
stan.model <- stan_model(file="model2.stan")
post <- sampling(stan.model, seed=2468, verbose=TRUE, refresh=10,
				  pars=c("mu","sig"), include=FALSE,
				  data=list(
					  nCond=nCond, nCov=nCov, nLab=nLab, nSubj=nSubj,nRow=nRow, nDepvar=nDepvar,
					  y=dat$value, subj=dat$SubjectID, lab=dat$LabID, 
					  XX=XX,
					  depvar=dat$EffectID, condition=dat$ConditionID, dv_scale=init.fcn()$sigma_d
				  ), 
				  init=init.fcn
				 )

# Post-process chains
# Step One: Impose identifiability constraints
x.match <- rep(c(-1,-1,1,1), nDepvar)
P3.fcn <- function(theta){
	P <- diag(3)
	P[1:2,2] <- theta[1:2]
	P[1:3,3] <- theta[3:5]
	return(P)
}
P3g.fcn <- function(theta){
	P <- diag(3)
	P[1:2,2] <- theta[1:2]
	return(P)
}
vcv.fcn <- function(theta,LL,OO,Pf){
	P <- Pf(theta)
	Pi <- solve(P)
	LLp <- LL%*%P
	OOp <- Pi%*%OO%*%t(Pi)
	return(list(LL=LLp, OO=OOp, VV=LLp%*%OOp%*%t(LLp), P=P, Pi=Pi))
}
d.idx2 <- seq(1, nCond*nDepvar, by=nCond)
d.idx3 <- seq(1, nCond*nDepvar, by=1)
dist3.fcn <- function(theta,LL,OO){
	tmp <- vcv.fcn(theta,LL,OO,P3.fcn)
	LLp <- tmp$LL
	OOp <- tmp$OO
	RRp <- cov2cor(OOp)
	d <- (mean(LLp[d.idx2,2]) - 0)^2 + (sd(LLp[d.idx2,2]) - 1)^2 +
		 (mean(LLp[d.idx3,3]) - 0)^2 + (sd(LLp[d.idx3,3]) - 1)^2 +
		 RRp[2,3]^2
	return(d)
}
dist3g.fcn <- function(theta,LL,OO){
	tmp <- vcv.fcn(theta,LL,OO,P3g.fcn)
	LLp <- tmp$LL
	OOp <- tmp$OO
	RRp <- cov2cor(OOp)
	d <- (mean(LLp[d.idx2,2]) - 0)^2 + (sd(LLp[d.idx2,2]) - 1)^2
	return(d)
}

aa <- as.array(post)
n.iter.post <- dim(aa)[[1]]
n.chains <- dim(aa)[[2]]
aa2 <- aa
theta3.id <- c(0,1,   0,0,1)
theta3g.id <- c(0,1)

for(j in 1:n.chains){
	for(i in 1:n.iter.post){	
		tmpl2 <- aa[i,j,grepl("lambda_beta1_d\\[",dimnames(aa)[[3]])]
		tmpl3 <- aa[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa)[[3]])]
		tmpp <- rbind(aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")])
		tmpo <- aa[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa)[[3]])]
		tmpR <- matrix(aa[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa)[[3]])], 3, 3)
	
		LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
		OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
		VV <- LL %*% OO %*% t(LL)
		if(i==1){ tmp.init <- theta3.id }else{ tmp.init <- oo$par }
		oo <- nlminb(tmp.init, dist3.fcn, LL=LL, OO=OO)		
		tmp <- vcv.fcn(oo$par, LL, OO, P3.fcn)
		tmppp <- solve(tmp$P) %*% tmpp
	
		aa2[i,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx2,2]
		aa2[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx3,3]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
		aa2[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])] <- sqrt(diag(tmp$OO))
		aa2[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa2)[[3]])] <- as.vector(tmp$OO)
		aa2[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa2)[[3]])] <- as.vector(cov2cor(tmp$OO))
	}
}

for(j in 1:n.chains){
	for(i in 1:n.iter.post){	
		tmpl2 <- aa[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa)[[3]])]
		tmpp <- rbind(aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")])
		tmpo <- aa[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa)[[3]])]
		tmpR <- matrix(aa[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa)[[3]])], 3, 3)
	
		LL <- cbind(1, rep(tmpl2,each=nCond), x.match)
		OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
		VV <- LL %*% OO %*% t(LL)
		if(i==1){ tmp.init <- theta3g.id }else{ tmp.init <- oo$par }
		oo <- nlminb(tmp.init, dist3g.fcn, LL=LL, OO=OO)		
		tmp <- vcv.fcn(oo$par, LL, OO, P3g.fcn)
		tmppp <- solve(tmp$P) %*% tmpp
	
		aa2[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx2,2]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
		aa2[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])] <- sqrt(diag(tmp$OO))
		aa2[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa2)[[3]])] <- as.vector(tmp$OO)
		aa2[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa2)[[3]])] <- as.vector(cov2cor(tmp$OO))
	}
}

# Step Two: Resolve within chain reflection invariance
chain.relabel <- function(LL, fac.idx, col.idx){
	# Erosheva and Curtis, Psychometrika (2017)
	qq <- max(fac.idx)
	if(length(col.idx)!=qq){print("error")}

	nu <- matrix(NA, nrow(LL), qq)
	LLr <- LL
	for(i in 1:qq){
		LLs <- LL[,col.idx[[i]]]
		nu1 <- rep(1, nrow(LL))
		crit <- TRUE
		while(crit){
			mm <- apply(LLs * nu1, 2, mean)
			ss <- apply(LLs * nu1, 2, sd)
			ll.plus <- colSums( apply(LLs, 1, dnorm, mean=mm, sd=ss, log=TRUE) )
			ll.minus <- colSums( apply(-LLs, 1, dnorm, mean=mm, sd=ss, log=TRUE) )
			
			nu0 <- nu1
			nu1 <- -1 + 2*(ll.plus > ll.minus)
			crit <- any(nu0!=nu1)		
		}
		
		nu[,i] <- nu1
		LLr[,col.idx[[i]]] <- LLs * nu1		
	}
	
	mm <- apply(LLr, 2, mean)
	ss <- apply(LLr, 2, sd)
	ll <- sum(colSums( apply(LLr, 1, dnorm, mean=mm, sd=ss, log=TRUE) ))
	out <- list(loadings=LLr, signs=nu, ll=ll)
}

aa3 <- aa2

factor.index <- rep(1:2, c(nDepvar,nDepvar*nCond))
column.index <- lapply(1:2, function(x){which(factor.index==x)})
for(j in 1:n.chains){
	tmpl2 <- aa2[,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])]
	tmpl3 <- aa2[,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])]
	tmpo <- aa2[,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])]
	
	tmpl2 <- tmpl2 * tmpo[,2]
	tmpl3 <- tmpl3 * tmpo[,3]
	
	LL <- cbind(tmpl2, tmpl3)
	tmp.relabel <- chain.relabel(LL, factor.index, column.index)
	
	tmp.idx <- which(rowSums(tmp.relabel$signs == -1) > 0)
	for(i in tmp.idx){	
		tmpl2 <- aa2[i,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])]
		tmpl3 <- aa2[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])]
		tmpp <- rbind(aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")])
		tmpo <- aa2[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])]
		tmpR <- matrix(aa2[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa2)[[3]])], 3, 3)

		flip1 <- tmp.relabel$signs[i,1] == -1
		flip2 <- tmp.relabel$signs[i,2] == -1
	
		if(flip1 | flip2){
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta3.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			if(flip2){ theta[3:5] <- -theta[3:5] }
			tmp <- vcv.fcn(theta,LL,OO,P3.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa3[i,j,grepl("lambda_beta1_d\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx2,2]
			aa3[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx3,3]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
			aa3[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa3)[[3]])] <- sqrt(diag(tmp$OO))
			aa3[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa3)[[3]])] <- as.vector(tmp$OO)
			aa3[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa3)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}	
	}
}

factor.index.g <- rep(1, c(nDepvar))
column.index.g <- lapply(1, function(x){which(factor.index==x)})
for(j in 1:n.chains){
	tmpl2 <- aa2[,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])]
	tmpo <- aa2[,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])]
	
	tmpl2 <- tmpl2 * tmpo[,2]
	
	LL <- cbind(tmpl2)
	tmp.relabel <- chain.relabel(LL, factor.index.g, column.index.g)
	
	tmp.idx <- which(rowSums(tmp.relabel$signs == -1) > 0)
	for(i in tmp.idx){	
		tmpl2 <- aa2[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])]
		tmpp <- rbind(aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")])
		tmpo <- aa2[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])]
		tmpR <- matrix(aa2[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa2)[[3]])], 3, 3)

		flip1 <- tmp.relabel$signs[i,1] == -1
	
		if(flip1){
			LL <- cbind(1, rep(tmpl2,each=nCond), x.match)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta3g.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			tmp <- vcv.fcn(theta,LL,OO,P3g.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa3[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx2,2]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
			aa3[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa3)[[3]])] <- sqrt(diag(tmp$OO))
			aa3[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa3)[[3]])] <- as.vector(tmp$OO)
			aa3[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa3)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}	
	}
}

# Step Three: Resolve between chain reflection invariance
tmp	<- aa3[,,"lambda_beta1_d[1]"]
chain.idx.lb1 <- as.vector(which(colMeans(tmp < 0) < 0.5))
tmp	<- aa3[,,"lambda_beta2_cd[1,1]"]
chain.idx.lb2 <- as.vector(which(colMeans(tmp < 0) < 0.5))
tmp	<- aa3[,,"lambda_gamma1_d[1]"]
chain.idx.lg1 <- as.vector(which(colMeans(tmp < 0) < 0.5))

aa4 <- aa3

for(j in 1:n.chains){	
	flip1 <- j %in% chain.idx.lb1
	flip2 <- j %in% chain.idx.lb2
	
	if(flip1 | flip2){			
		for(i in 1:n.iter.post){
			tmpl2 <- aa3[i,j,grepl("lambda_beta1_d\\[",dimnames(aa3)[[3]])]
			tmpl3 <- aa3[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa3)[[3]])]
			tmpp <- rbind(aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")])
			tmpo <- aa3[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa3)[[3]])]
			tmpR <- matrix(aa3[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa3)[[3]])], 3, 3)
	
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta3.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			if(flip2){ theta[3:5] <- -theta[3:5] }
			tmp <- vcv.fcn(theta,LL,OO,P3.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa4[i,j,grepl("lambda_beta1_d\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx2,2]
			aa4[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx3,3]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
			aa4[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa4)[[3]])] <- sqrt(diag(tmp$OO))
			aa4[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa4)[[3]])] <- as.vector(tmp$OO)
			aa4[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa4)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}
	}
}

for(j in 1:n.chains){	
	flip1 <- j %in% chain.idx.lg1
	
	if(flip1){			
		for(i in 1:n.iter.post){
			tmpl2 <- aa3[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa3)[[3]])]
			tmpp <- rbind(aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")])
			tmpo <- aa3[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa3)[[3]])]
			tmpR <- matrix(aa3[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa3)[[3]])], 3, 3)
	
			LL <- cbind(1, rep(tmpl2,each=nCond), x.match)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta3g.id
			if(flip1){ theta[1:2] <- -theta[1:2] }
			tmp <- vcv.fcn(theta,LL,OO,P3g.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa4[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx2,2]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
			aa4[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa4)[[3]])] <- sqrt(diag(tmp$OO))
			aa4[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa4)[[3]])] <- as.vector(tmp$OO)
			aa4[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa4)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}
	}
}

# Plot output 
cond.orig <- cond
cond <- c("Left, large", "Left, small", "Right, large", "Right, small")
cond.order <- cond[c(2,1,4,3)]

fun.alpha <- function(y,alpha=0.05){ 
	out <- quantile(y,probs=c(alpha/2,0.5,1-alpha/2))
	names(out) <- c("ymin","y","ymax")
	out
}
f95 <- function(y){ fun.alpha(y,0.05) }
f50 <- function(y){ fun.alpha(y,0.50) }
ppp <- c(0.025, 0.25,   0.50,   0.75, 0.975)
qqq <- qnorm(ppp)

cc <- matrix(0, nDepvar, nCond*nDepvar)
for(i in 1:nDepvar){
	tmpj <- (i-1)*nCond + 1
	cc[i,tmpj:(tmpj+nCond-1)] <- c(-1,1,1,-1)
}
modk <- function(x,k){ tmp <- x%%k; ifelse(tmp==0,k,tmp) }

tmp1 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- apply(aa4[,,grepl("lambda_gamma1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,rep(1:nDepvar, each=nCond)]
tmp3 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmpS <- apply(aa4[,,grepl("omega_phi_gamma_f", dimnames(aa4)[[3]])], 3, identity)
for(i in 1:ncol(tmp1)){ tmp1[,i] <- 1 * tmpS[,1] }
for(i in 1:ncol(tmp2)){ tmp2[,i] <- tmp2[,i] * tmpS[,2] }
for(i in 1:ncol(tmp3)){ tmp3[,i] <- x.match[i] * tmpS[,3] }

tmpgg <- rbind( melt(tmp1),melt(tmp2),melt(tmp3) )
tmp.name <- c("Intercept", "Dependent Measure", "Variate (Fully Constrained)")
tmpgg$Factor <- rep(tmp.name, c(length(tmp1),length(tmp2),length(tmp3)) )
tmpgg$Factor <- factor(tmpgg$Factor, levels=unique(tmp.name))
tmpgg$Effect <- c( rep(study,each=nCond*nrow(tmp1)), rep(study,each=nCond*nrow(tmp2)), rep(study,each=nCond*nrow(tmp3)) )
tmpgg$Effect <- factor(tmpgg$Effect, levels=rev(study))
tmpc <- rep(cond, each=nrow(tmp3))
tmpgg$Condition <- c(rep(tmpc,nDepvar), rep(tmpc,nDepvar), rep(tmpc,nDepvar) )
tmpgg$Condition <- factor(tmpgg$Condition, levels=cond.order)
tmp1 <- data.frame(Factor=factor(tmp.name,levels=tmp.name), yy=c(NA,0,0))
tmp2 <- data.frame(Factor=factor(rep(tmp.name,each=2), levels=tmp.name),
						Effect=tmpgg$Effect[1], Condition=tmpgg$Condition[1],
						value=c(40,48,   -12,12,    -12,12)/10)
plot4 <- ggplot(tmpgg, aes(x=Effect,y=value*10,col=Condition)) + coord_flip() + 
		facet_grid(~Factor, scales="free_x") +
		stat_summary(fun.y=median, geom="point", position=position_dodge(width=-2/3)) + 
		stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-2/3), show.legend=FALSE) +
		stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-2/3)) +
		xlab("Dependent Measure") + ylab("Loading (ms)") + 
		geom_hline(data=tmp1,aes(yintercept=yy),linetype=2,size=0.25) +
		geom_point(data=tmp2,col=NA) +
		theme(legend.position="bottom")
		
ggsave("graph_paper_rrr4.pdf", plot=plot4, device=cairo_pdf, width=4/3*4.45, height=4/3*2.35)
