# Load packages and read in data
library(ggplot2)
library(reshape2)
library(abind)
library(rstan)
options(mc.cores = parallel::detectCores())

processed.data <- readRDS("manylabs.data.rds")
YY <- processed.data$YY
TT <- processed.data$TT
lab <- processed.data$lab.id
study <- processed.data$study.info[,1]
var.type <- processed.data$study.info[,2]

# Prepare data for Stan
dat <- melt(YY)
colnames(dat)[1:2] <- c("SubjectID","EffectID")
dat$Treatment <- melt(TT)$value + 1
dat$Lab <- lab
dat$LabID <- as.numeric(dat$Lab)
dat$VarType <- var.type[dat$EffectID]
dat <- dat[!is.na(dat$value),]

# Scalars for stan
nCond <- 2		# All DVs have two conditions
nCatO <- 9		# Both ordinal DVs are 1:9
nLab <- max(dat$LabID)
nSubj <- max(dat$SubjectID)
nRowC <- sum(dat$VarType=="C")
nRowB <- sum(dat$VarType=="B")
nRowO <- sum(dat$VarType=="O")
nDepvar <- length(unique(dat$EffectID))
nDepvarC <- length(unique(dat$EffectID[dat$VarType=="C"]))
nDepvarB <- length(unique(dat$EffectID[dat$VarType=="B"]))
nDepvarO <- length(unique(dat$EffectID[dat$VarType=="O"]))

# Continuous for stan
datC <- dat[dat$VarType=="C",]
datC$EffectID2 <- as.numeric(as.factor(datC$EffectID))
s.pool <- rep(NA, max(datC$EffectID2))
for(i in 1:length(s.pool)){
	sel <- datC$EffectID2==i
	tmpv <- by(datC$value[sel], list(datC$Lab[sel], datC$Treatment[sel]), var)
	tmpn <- by(datC$value[sel], list(datC$Lab[sel], datC$Treatment[sel]), length)
	s.pool[i] <- sqrt( sum((tmpn-1)*tmpv, na.rm=TRUE) / sum(tmpn-1, na.rm=TRUE) )
}

# Binary for stan
datB <- dat[dat$VarType=="B",]
datB$EffectID2 <- as.numeric(as.factor(datB$EffectID)) + nDepvarC

# Ordinal for stan
datO <- dat[dat$VarType=="O",]
datO$EffectID2 <- as.numeric(as.factor(datO$EffectID)) + nDepvarC + nDepvarB

study.orig <- study
var.type.orig <- var.type
study <- study[c(unique(datC$EffectID), unique(datB$EffectID), unique(datO$EffectID))]
var.type <- var.type[c(unique(datC$EffectID), unique(datB$EffectID), unique(datO$EffectID))]
Anch_ind <- 1*grepl("Anchoring", study)
nAnch <- sum(Anch_ind)

# Compile model and run sampler
stan.model <- stan_model(file="model1.stan")
post <- sampling(stan.model, seed=1234, verbose=TRUE, refresh=10,
				  pars=c("muC","sigC","muB","muO","log_likO"), include=FALSE,
				  data=list(
					  nCond=nCond, nCatO=nCatO, nLab=nLab, nSubj=nSubj,
					  nRowC=nRowC, nRowB=nRowB, nRowO=nRowO, 
					  nDepvar=nDepvar, nDepvarC=nDepvarC, nDepvarB=nDepvarB, nDepvarO=nDepvarO,
					  yC=datC$value, subjC=datC$SubjectID, labC=datC$LabID,
					  depvarC=datC$EffectID2, conditionC=datC$Treatment,  dv_scale=s.pool,
					  yB=datB$value, subjB=datB$SubjectID, labB=datB$LabID,
					  depvarB=datB$EffectID2, conditionB=datB$Treatment,
					  yO=datO$value, subjO=datO$SubjectID, labO=datO$LabID,
					  depvarO=datO$EffectID2, conditionO=datO$Treatment,
					  Anch_ind=Anch_ind
				  ), 
				  init_r=1
				 )
				 
# Post-process chains
# Step One: Impose identifiability constraints
P7.fcn <- function(theta){
	P <- diag(7)
	P[c(1,2,4,5,6),2] <- theta[c(1,2,3,3,4)]
	P[,3] <- theta[5:11]
	P[c(4,5,6),6] <- theta[c(12,12,13)]
	P[c(4,5,6,7),7] <- theta[c(14,15,16,17)]
	return(P)
}
P5.fcn <- function(theta){
	P <- diag(5)
	P[c(1,2,4),2] <- theta[1:3]
	P[,3] <- theta[4:8]
	P[4:5,5] <- theta[9:10]
	return(P)
}
vcv.fcn <- function(theta,LL,OO,Pf){
	P <- Pf(theta)
	Pi <- solve(P)
	LLp <- LL%*%P
	OOp <- Pi%*%OO%*%t(Pi)
	return(list(LL=LLp, OO=OOp, VV=LLp%*%OOp%*%t(LLp), P=P, Pi=Pi))
}
d.idx2 <- seq(1, nCond*nDepvar, by=nCond)
d.idx3 <- seq(1, nCond*nDepvar, by=1)
d.idx6 <- seq(1, nCond*nAnch, by=nCond)
d.idx7 <- seq(1, nCond*nAnch, by=1)
dist7.fcn <- function(theta,LL,OO){
	tmp <- vcv.fcn(theta,LL,OO,P7.fcn)
	LLp <- tmp$LL
	OOp <- tmp$OO
	d <- (mean(LLp[d.idx2,2]) - 0)^2 + (mean(LLp[d.idx3,3]) - 0)^2 +
		 (mean(LLp[d.idx6,6]) - 0)^2 + (mean(LLp[d.idx7,7]) - 0)^2 +
		 (sd(LLp[d.idx2,2]) - 1)^2 + (sd(LLp[d.idx3,3]) - 1)^2 +
		 (sd(LLp[d.idx6,6]) - 1)^2 + (sd(LLp[d.idx7,7]) - 1)^2 +
		 OOp[2,3]^2 + (OOp[2,4] + OOp[2,5])^2 + OOp[2,6]^2 + 
		 OOp[3,4]^2 + OOp[3,5]^2 + OOp[3,6]^2 + OOp[3,7]^2 +
		 (OOp[4,7] - OOp[5,7])^2 +
		 OOp[6,7]^2
	return(d)
}
dist5.fcn <- function(theta,LL,OO){
	tmp <- vcv.fcn(theta,LL,OO,P5.fcn)
	LLp <- tmp$LL
	OOp <- tmp$OO
	d <- (mean(LLp[d.idx2,2]) - 0)^2 + (mean(LLp[d.idx3,3]) - 0)^2 + 
		 (mean(LLp[d.idx7,5]) - 0)^2 + 
		 (sd(LLp[d.idx2,2]) - 1)^2 + (sd(LLp[d.idx3,3]) - 1)^2 +
		 (sd(LLp[d.idx7,5]) - 1)^2 + 
		 OOp[2,3]^2 + OOp[2,4]^2 +	 
		 OOp[3,4]^2 + OOp[3,5]^2
		 return(d)
}

aa <- as.array(post)
n.iter.post <- dim(aa)[[1]]
n.chains <- dim(aa)[[2]]
aa2 <- aa
theta7.id <- c(0,1,0,0,   0,0,1,0,0,0,0,   0,1,   0,0,0,1)
theta5.id <- c(0,1,0,   0,0,1,0,0,   0,1)

for(j in 1:n.chains){
	for(i in 1:n.iter.post){	
		tmpl2 <- aa[i,j,grepl("lambda_beta1_d\\[",dimnames(aa)[[3]])]
		tmpl3 <- aa[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa)[[3]])]
		tmpl6 <- aa[i,j,grepl("lambda_beta3_d\\[",dimnames(aa)[[3]])]
		tmpl7 <- aa[i,j,grepl("lambda_beta4_cd\\[",dimnames(aa)[[3]])]
		tmpp <- rbind(aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",4]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",5]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",6]")],
					   aa[i,j,paste0("phi_beta_lf[", 1:nLab, ",7]")])
		tmpo <- aa[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa)[[3]])]
		tmpR <- matrix(aa[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa)[[3]])], 7, 7)
	
		LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3, 
					rep(Anch_ind,each=nCond) * rep(1:0, nDepvar),
					rep(Anch_ind,each=nCond) * rep(0:1, nDepvar),
					rep(tmpl6,each=nCond), tmpl7)
		OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
		VV <- LL %*% OO %*% t(LL)
		if(i==1){ tmp.init <- theta7.id }else{ tmp.init <- oo$par }
		oo <- nlminb(tmp.init, dist7.fcn, LL=LL, OO=OO, control=list(iter.max=10000, eval.max=100000))		
		tmp <- vcv.fcn(oo$par, LL, OO, P7.fcn)
		tmppp <- solve(tmp$P) %*% tmpp
	
		aa2[i,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx2,2]
		aa2[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx3,3]
		aa2[i,j,grepl("lambda_beta3_d\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx2,6]
		aa2[i,j,grepl("lambda_beta4_cd\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx3,7]
		aa2[i,j,grepl("lambda_beta3_d_raw\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx6,6]
		aa2[i,j,grepl("lambda_beta4_cd_raw\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx7,7]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",4]")] <- tmppp[4,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",5]")] <- tmppp[5,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",6]")] <- tmppp[6,]
		aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",7]")] <- tmppp[7,]
		aa2[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])] <- sqrt(diag(tmp$OO))
		aa2[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa2)[[3]])] <- as.vector(tmp$OO)
		aa2[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa2)[[3]])] <- as.vector(cov2cor(tmp$OO))
	}
}

for(j in 1:n.chains){
	for(i in 1:n.iter.post){	
		tmpl2 <- aa[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa)[[3]])]
		tmpl3 <- aa[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa)[[3]])]
		tmpl5 <- aa[i,j,grepl("lambda_gamma3_cd\\[",dimnames(aa)[[3]])]
		tmpp <- rbind(aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",4]")],
					   aa[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",5]")])
		tmpo <- aa[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa)[[3]])]
		tmpR <- matrix(aa[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa)[[3]])], 5, 5)
	
		LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3, rep(Anch_ind,each=nCond), tmpl5)
		OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
		VV <- LL %*% OO %*% t(LL)
		if(i==1){ tmp.init <- theta5.id }else{ tmp.init <- oo$par }
		oo <- nlminb(tmp.init, dist5.fcn, LL=LL, OO=OO, control=list(iter.max=10000, eval.max=100000))		
		tmp <- vcv.fcn(oo$par, LL, OO, P5.fcn)
		tmppp <- solve(tmp$P) %*% tmpp
	
		aa2[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx2,2]
		aa2[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx3,3]
		aa2[i,j,grepl("lambda_gamma3_cd\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx3,5]
		aa2[i,j,grepl("lambda_gamma3_cd_raw\\[",dimnames(aa2)[[3]])] <- tmp$LL[d.idx7,5]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",4]")] <- tmppp[4,]
		aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",5]")] <- tmppp[5,]
		aa2[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])] <- sqrt(diag(tmp$OO))
		aa2[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa2)[[3]])] <- as.vector(tmp$OO)
		aa2[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa2)[[3]])] <- as.vector(cov2cor(tmp$OO))
	}
}

# Step Two: Resolve within chain reflection invariance
chain.relabel <- function(LL, fac.idx, col.idx){
	# Erosheva and Curtis, Psychometrika (2017)
	qq <- max(fac.idx)
	if(length(col.idx)!=qq){print("error")}

	nu <- matrix(NA, nrow(LL), qq)
	LLr <- LL
	for(i in 1:qq){
		LLs <- LL[,col.idx[[i]]]
		nu1 <- rep(1, nrow(LL))
		crit <- TRUE
		while(crit){
			mm <- apply(LLs * nu1, 2, mean)
			ss <- apply(LLs * nu1, 2, sd)
			ll.plus <- colSums( apply(LLs, 1, dnorm, mean=mm, sd=ss, log=TRUE) )
			ll.minus <- colSums( apply(-LLs, 1, dnorm, mean=mm, sd=ss, log=TRUE) )
			
			nu0 <- nu1
			nu1 <- -1 + 2*(ll.plus > ll.minus)
			crit <- any(nu0!=nu1)		
		}
		
		nu[,i] <- nu1
		LLr[,col.idx[[i]]] <- LLs * nu1		
	}
	
	mm <- apply(LLr, 2, mean)
	ss <- apply(LLr, 2, sd)
	ll <- sum(colSums( apply(LLr, 1, dnorm, mean=mm, sd=ss, log=TRUE) ))
	out <- list(loadings=LLr, signs=nu, ll=ll)
}

aa3 <- aa2

factor7.index <- rep(1:4, c(nDepvar,nDepvar*nCond,nAnch,nAnch*nCond))
column7.index <- lapply(1:4, function(x){which(factor7.index==x)})
factor5.index <- rep(1:3, c(nDepvar,nDepvar*nCond,nAnch*nCond))
column5.index <- lapply(1:3, function(x){which(factor5.index==x)})
for(j in 1:n.chains){
	tmpl2 <- aa2[,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])]
	tmpl3 <- aa2[,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])]
	tmpl6 <- aa2[,j,grepl("lambda_beta3_d_raw\\[",dimnames(aa2)[[3]])]
	tmpl7 <- aa2[,j,grepl("lambda_beta4_cd_raw\\[",dimnames(aa2)[[3]])]
	tmpo <- aa2[,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])]
	
	tmpl2 <- tmpl2 * tmpo[,2]
	tmpl3 <- tmpl3 * tmpo[,3]
	tmpl6 <- tmpl6 * tmpo[,6]
	tmpl7 <- tmpl7 * tmpo[,7]
	
	LL <- cbind(tmpl2, tmpl3, tmpl6, tmpl7)
	tmp.relabel <- chain.relabel(LL, factor7.index, column7.index)
	
	tmp.idx <- which(rowSums(tmp.relabel$signs == -1) > 0)
	for(i in tmp.idx){	
		tmpl2 <- aa2[i,j,grepl("lambda_beta1_d\\[",dimnames(aa2)[[3]])]
		tmpl3 <- aa2[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa2)[[3]])]
		tmpl6 <- aa2[i,j,grepl("lambda_beta3_d\\[",dimnames(aa2)[[3]])]
		tmpl7 <- aa2[i,j,grepl("lambda_beta4_cd\\[",dimnames(aa2)[[3]])]
		tmpp <- rbind(aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",4]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",5]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",6]")],
					   aa2[i,j,paste0("phi_beta_lf[", 1:nLab, ",7]")])
		tmpo <- aa2[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa2)[[3]])]
		tmpR <- matrix(aa2[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa2)[[3]])], 7, 7)

		flip1 <- tmp.relabel$signs[i,1] == -1
		flip2 <- tmp.relabel$signs[i,2] == -1
		flip3 <- tmp.relabel$signs[i,3] == -1
		flip4 <- tmp.relabel$signs[i,4] == -1
	
		if(flip1 | flip2 | flip3 | flip4){
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3, 
						rep(Anch_ind,each=nCond) * rep(1:0, nDepvar),
						rep(Anch_ind,each=nCond) * rep(0:1, nDepvar),
						rep(tmpl6,each=nCond), tmpl7)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta7.id
			if(flip1){ theta[1:4] <- -theta[1:4] }
			if(flip2){ theta[5:11] <- -theta[5:11] }
			if(flip3){ theta[12:13] <- -theta[12:13] }
			if(flip4){ theta[14:17] <- -theta[14:17] }
			tmp <- vcv.fcn(theta,LL,OO,P7.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa3[i,j,grepl("lambda_beta1_d\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx2,2]
			aa3[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx3,3]
			aa3[i,j,grepl("lambda_beta3_d\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx2,6]
			aa3[i,j,grepl("lambda_beta4_cd\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx3,7]
			aa3[i,j,grepl("lambda_beta3_d_raw\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx6,6]
			aa3[i,j,grepl("lambda_beta4_cd_raw\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx7,7]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",4]")] <- tmppp[4,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",5]")] <- tmppp[5,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",6]")] <- tmppp[6,]
			aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",7]")] <- tmppp[7,]
			aa3[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa3)[[3]])] <- sqrt(diag(tmp$OO))
			aa3[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa3)[[3]])] <- as.vector(tmp$OO)
			aa3[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa3)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}	
	}
}

for(j in 1:n.chains){
	tmpl2 <- aa2[,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])]
	tmpl3 <- aa2[,j,grepl("lambda_gamma2_cd\\[",dimnames(aa2)[[3]])]
	tmpl5 <- aa2[,j,grepl("lambda_gamma3_cd_raw\\[",dimnames(aa2)[[3]])]
	tmpo <- aa2[,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])]
	
	tmpl2 <- tmpl2 * tmpo[,2]
	tmpl3 <- tmpl3 * tmpo[,3]
	tmpl5 <- tmpl5 * tmpo[,5]
	
	LL <- cbind(tmpl2, tmpl3, tmpl5)
	tmp.relabel <- chain.relabel(LL, factor5.index, column5.index)
	
	tmp.idx <- which(rowSums(tmp.relabel$signs == -1) > 0)
	for(i in tmp.idx){	
		tmpl2 <- aa2[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa2)[[3]])]
		tmpl3 <- aa2[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa2)[[3]])]
		tmpl5 <- aa2[i,j,grepl("lambda_gamma3_cd\\[",dimnames(aa2)[[3]])]
		tmpp <- rbind(aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",4]")],
					   aa2[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",5]")])
		tmpo <- aa2[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa2)[[3]])]
		tmpR <- matrix(aa2[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa2)[[3]])], 5, 5)

		flip1 <- tmp.relabel$signs[i,1] == -1
		flip2 <- tmp.relabel$signs[i,2] == -1
		flip3 <- tmp.relabel$signs[i,3] == -1
	
		if(flip1 | flip2 | flip3){
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3, rep(Anch_ind,each=nCond), tmpl5)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta5.id
			if(flip1){ theta[1:3] <- -theta[1:3] }
			if(flip2){ theta[4:8] <- -theta[4:8] }
			if(flip3){ theta[9:10] <- -theta[9:10] }
			tmp <- vcv.fcn(theta,LL,OO,P5.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa3[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx2,2]
			aa3[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx3,3]
			aa3[i,j,grepl("lambda_gamma3_cd\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx3,5]
			aa3[i,j,grepl("lambda_gamma3_cd_raw\\[",dimnames(aa3)[[3]])] <- tmp$LL[d.idx7,5]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",4]")] <- tmppp[4,]
			aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",5]")] <- tmppp[5,]
			aa3[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa3)[[3]])] <- sqrt(diag(tmp$OO))
			aa3[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa3)[[3]])] <- as.vector(tmp$OO)
			aa3[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa3)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}	
	}
}

# Step Three: Resolve between chain reflection invariance
tmp	<- aa3[,,"lambda_beta1_d[6]"]
chain.idx.lb1 <- as.vector(which(colMeans(tmp < 0) > 0.5))
tmp	<- aa3[,,"lambda_beta2_cd[1,6]"]
chain.idx.lb2 <- as.vector(which(colMeans(tmp < 0) > 0.5))
tmp	<- aa3[,,"lambda_beta3_d_raw[3]"]
chain.idx.lb3 <- as.vector(which(colMeans(tmp < 0) < 0.5))
tmp	<- aa3[,,"lambda_beta4_cd_raw[1,3]"]
chain.idx.lb4 <- as.vector(which(colMeans(tmp < 0) < 0.5))
tmp	<- aa3[,,"lambda_gamma1_d[6]"]
chain.idx.lg1 <- as.vector(which(colMeans(tmp < 0) > 0.5))
tmp	<- aa3[,,"lambda_gamma2_cd[1,6]"]
chain.idx.lg2 <- as.vector(which(colMeans(tmp < 0) > 0.5))
tmp	<- aa3[,,"lambda_gamma3_cd_raw[1,3]"]
chain.idx.lg3 <- as.vector(which(colMeans(tmp < 0) < 0.5))

aa4 <- aa3

for(j in 1:n.chains){	
	flip1 <- j %in% chain.idx.lb1
	flip2 <- j %in% chain.idx.lb2
	flip3 <- j %in% chain.idx.lb3
	flip4 <- j %in% chain.idx.lb4
	
	if(flip1 | flip2 | flip3 | flip4){			
		for(i in 1:n.iter.post){
			tmpl2 <- aa3[i,j,grepl("lambda_beta1_d\\[",dimnames(aa3)[[3]])]
			tmpl3 <- aa3[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa3)[[3]])]
			tmpl6 <- aa3[i,j,grepl("lambda_beta3_d\\[",dimnames(aa3)[[3]])]
			tmpl7 <- aa3[i,j,grepl("lambda_beta4_cd\\[",dimnames(aa3)[[3]])]
			tmpp <- rbind(aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",4]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",5]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",6]")],
						   aa3[i,j,paste0("phi_beta_lf[", 1:nLab, ",7]")])
			tmpo <- aa3[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa3)[[3]])]
			tmpR <- matrix(aa3[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa3)[[3]])], 7, 7)
	
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3, 
						rep(Anch_ind,each=nCond) * rep(1:0, nDepvar),
						rep(Anch_ind,each=nCond) * rep(0:1, nDepvar),
						rep(tmpl6,each=nCond), tmpl7)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta7.id
			if(flip1){ theta[1:4] <- -theta[1:4] }
			if(flip2){ theta[5:11] <- -theta[5:11] }
			if(flip3){ theta[12:13] <- -theta[12:13] }
			if(flip4){ theta[14:17] <- -theta[14:17] }
			tmp <- vcv.fcn(theta,LL,OO,P7.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa4[i,j,grepl("lambda_beta1_d\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx2,2]
			aa4[i,j,grepl("lambda_beta2_cd\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx3,3]
			aa4[i,j,grepl("lambda_beta3_d\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx2,6]
			aa4[i,j,grepl("lambda_beta4_cd\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx3,7]
			aa4[i,j,grepl("lambda_beta3_d_raw\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx6,6]
			aa4[i,j,grepl("lambda_beta4_cd_raw\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx7,7]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",1]")] <- tmppp[1,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",2]")] <- tmppp[2,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",3]")] <- tmppp[3,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",4]")] <- tmppp[4,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",5]")] <- tmppp[5,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",6]")] <- tmppp[6,]
			aa4[i,j,paste0("phi_beta_lf[", 1:nLab, ",7]")] <- tmppp[7,]
			aa4[i,j,grepl("omega_phi_beta_f\\[",dimnames(aa4)[[3]])] <- sqrt(diag(tmp$OO))
			aa4[i,j,grepl("Omega_phi_beta_ff\\[",dimnames(aa4)[[3]])] <- as.vector(tmp$OO)
			aa4[i,j,grepl("Rho_phi_beta_ff\\[",dimnames(aa4)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}
	}
}

for(j in 1:n.chains){	
	flip1 <- j %in% chain.idx.lg1
	flip2 <- j %in% chain.idx.lg2
	flip3 <- j %in% chain.idx.lg3
	
	if(flip1 | flip2 | flip3){			
		for(i in 1:n.iter.post){
			tmpl2 <- aa3[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa3)[[3]])]
			tmpl3 <- aa3[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa3)[[3]])]
			tmpl5 <- aa3[i,j,grepl("lambda_gamma3_cd\\[",dimnames(aa3)[[3]])]
			tmpp <- rbind(aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",4]")],
						   aa3[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",5]")])
			tmpo <- aa3[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa3)[[3]])]
			tmpR <- matrix(aa3[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa3)[[3]])], 5, 5)
	
			LL <- cbind(1, rep(tmpl2,each=nCond), tmpl3, rep(Anch_ind,each=nCond), tmpl5)
			OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
			VV <- LL %*% OO %*% t(LL)
		
			theta <- theta5.id
			if(flip1){ theta[1:3] <- -theta[1:3] }
			if(flip2){ theta[4:8] <- -theta[4:8] }
			if(flip3){ theta[9:10] <- -theta[9:10] }
			tmp <- vcv.fcn(theta,LL,OO,P5.fcn)
			tmppp <- solve(tmp$P) %*% tmpp
		
			aa4[i,j,grepl("lambda_gamma1_d\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx2,2]
			aa4[i,j,grepl("lambda_gamma2_cd\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx3,3]
			aa4[i,j,grepl("lambda_gamma3_cd\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx3,5]
			aa4[i,j,grepl("lambda_gamma3_cd_raw\\[",dimnames(aa4)[[3]])] <- tmp$LL[d.idx7,5]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",1]")] <- tmppp[1,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",2]")] <- tmppp[2,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",3]")] <- tmppp[3,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",4]")] <- tmppp[4,]
			aa4[i,j,paste0("phi_gamma_sf[", 1:nSubj, ",5]")] <- tmppp[5,]
			aa4[i,j,grepl("omega_phi_gamma_f\\[",dimnames(aa4)[[3]])] <- sqrt(diag(tmp$OO))
			aa4[i,j,grepl("Omega_phi_gamma_ff\\[",dimnames(aa4)[[3]])] <- as.vector(tmp$OO)
			aa4[i,j,grepl("Rho_phi_gamma_ff\\[",dimnames(aa4)[[3]])] <- as.vector(cov2cor(tmp$OO))
		}
	}
}

# Plot output
study.orig <- study
study <- c(
	"Anchoring:\nDistance from SF to NYC",
	"Anchoring:\nPopulation of Chicago",
	"Anchoring:\nHeight of Mount Everest",
	"Anchoring:\nUS Babies Born Per Day",
	"Retrospective\nGambler Fallacy",
	"Flag Priming",
	"Currency Priming",
	"Imagined Contact",
	"Sex Differences in\nImplicit Math Attitudes",
	"Gain vs. Loss Framing",
	"Low vs. High\nCategory Scales",
	"Norm of Reciprocity",
	"Allowed vs. Forbidden",
	"Sunk Costs",
	"Quote Attribution")

fun.alpha <- function(y,alpha=0.05){ 
	out <- quantile(y,probs=c(alpha/2,0.5,1-alpha/2))
	names(out) <- c("ymin","y","ymax")
	out
}
f95 <- function(y){ fun.alpha(y,0.05) }
f50 <- function(y){ fun.alpha(y,0.50) }
ppp <- c(0.025, 0.25,   0.50,   0.75, 0.975)
qqq <- qnorm(ppp)

cc <- matrix(0, nDepvar, nCond*nDepvar)
for(i in 1:nDepvar){
	tmpj <- (i-1)*nCond + 1
	cc[i,tmpj:(tmpj+nCond-1)] <- c(-1,1)
}
modk <- function(x,k){ tmp <- x%%k; ifelse(tmp==0,k,tmp) }
cond <- c("Condition 1", "Condition 2")

tmp.idx0 <- 1:nDepvar
tmp.idx1 <- seq(1, nCond*nDepvar, by=nCond)
tmp.idx2 <- rep(tmp.idx0, each=nCond)
tmp.idx3 <- rep(tmp.idx0, each=nCond) + rep(c(0,nDepvar), nDepvar)
tmp2 <- apply(aa4[,,grepl("lambda_beta1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,tmp.idx2]
tmp3 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp6 <- apply(aa4[,,grepl("lambda_beta3_d\\[", dimnames(aa4)[[3]])], 3, identity)
tmp6 <- tmp6[,tmp.idx2]
tmp7 <- apply(aa4[,,grepl("lambda_beta4_cd\\[", dimnames(aa4)[[3]])], 3, identity)
tmpTc <- apply(aa4[,,grepl("Omega_phi_beta_ff", dimnames(aa4)[[3]])], 3, identity)
tmpTe <- apply(aa4[,,grepl("Omega_eta_dcc", dimnames(aa4)[[3]])], 3, identity)
tmpov1 <- tmpov2 <- array(NA, c(nrow(tmp2), ncol(tmp2), ncol(tmp2)))
tmpoc1 <- tmpoc2 <- array(NA, c(nrow(tmp2), nrow(cc), nrow(cc)))
for(i in 1:nrow(tmp2)){		
	tmpa <- cbind(1, tmp2[i,], tmp3[i,],
		rep(Anch_ind,each=nCond) * rep(1:0, nDepvar),
		rep(Anch_ind,each=nCond) * rep(0:1, nDepvar),
		tmp6[i,], tmp7[i,])
	tmpb <- matrix(tmpTc[i,], 7, 7)
	tmpc <- tmpa %*% tmpb %*% t(tmpa)
	
	tmp.eta <- matrix(0,nCond*nDepvar,nCond*nDepvar)
	tmp.Te <-  tmpTe[i,]
	for(j in 1:nDepvar){
		tmp <- matrix(tmp.Te[grepl(paste0("\\[",j,","), names(tmp.Te))], nCond, nCond)
		tmp.eta[(1:nCond)+(j-1)*nCond,(1:nCond)+(j-1)*nCond] <- tmp
	}
	
	tmpov1[i,,] <- tmpc
	tmpov2[i,,] <- tmp.eta
	tmpoc1[i,,] <- cc %*% tmpc %*% t(cc)
	tmpoc2[i,,] <- cc %*% tmp.eta %*% t(cc)	
}

tmp.idx0 <- 1:nDepvar
tmp.idx1 <- seq(1, nCond*nDepvar, by=nCond)
tmp.idx2 <- rep(tmp.idx0, each=nCond)
tmp.idx3 <- rep(tmp.idx0, each=nCond) + rep(c(0,nDepvar), nDepvar)
tmp2 <- apply(aa4[,,grepl("lambda_gamma1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,tmp.idx2]
tmp3 <- apply(aa4[,,grepl("lambda_gamma2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp5 <- apply(aa4[,,grepl("lambda_gamma3_cd\\[", dimnames(aa4)[[3]])], 3, identity)
tmpTc <- apply(aa4[,,grepl("Omega_phi_gamma_ff", dimnames(aa4)[[3]])], 3, identity)
tmps <- apply(aa4[,,grepl("sigma", dimnames(aa4)[[3]])], 3, identity)
tmpov3 <- array(NA, c(nrow(tmp2), ncol(tmp2), ncol(tmp2)))
tmpoc3 <- array(NA, c(nrow(tmp2), nrow(cc), nrow(cc)))
for(i in 1:nrow(tmp2)){		
	tmpa <- cbind(1, tmp2[i,], tmp3[i,], rep(Anch_ind,each=nCond), tmp5[i,])
	tmpb <- matrix(tmpTc[i,], 5, 5)
	tmpc <- tmpa %*% tmpb %*% t(tmpa)
	
	tmpov3[i,,] <- tmpc
	tmpoc3[i,,] <- cc %*% tmpc %*% t(cc)
}

tmp <- tmpov1 + tmpov2 + tmpov3
tmp2 <- t(apply(tmp, 1, function(x){diag(abs(cc) %*% diag(diag(x)) %*% t(abs(cc)))}))
tmp3 <- apply(tmp2, 2, median)
tmp4 <- study[order(tmp3)]
idx <- grep("Anchoring", tmp4)
study.order <- tmp4[c(setdiff(1:length(study),idx),idx)]

tmp <- abind(tmpov1+tmpov2, tmpov3, along=0)
tmp2 <- melt(tmp)
tmp2$Level <- c("Lab", "Subject")[tmp2$Var1]
tmp2$Effect1 <- study[ceiling(tmp2$Var3/nCond)]
tmp2$Effect2 <- study[ceiling(tmp2$Var4/nCond)]
tmp2$Cond1 <- cond[modk(tmp2$Var3, nCond)]
tmp2$Cond2 <- cond[modk(tmp2$Var4, nCond)]
sel <- tmp2$Effect1==tmp2$Effect2 & tmp2$Cond1==tmp2$Cond2  
tmp3a <- tmp2[sel,c("value", "Level","Effect1","Cond1")]
colnames(tmp3a)[3:4] <- c("Effect","Condition")

tmp <- abind(tmpoc1+tmpoc2, tmpoc3, along=0)
tmp2 <- melt(tmp)
tmp2$Level <- c("Lab", "Subject")[tmp2$Var1]
tmp2$Effect1 <- study[tmp2$Var3]
tmp2$Effect2 <- study[tmp2$Var4]
sel <- tmp2$Effect1==tmp2$Effect2 
tmp3b <- tmp2[sel,c("value", "Level","Effect1")]
colnames(tmp3b)[3] <- "Effect"
tmp3b$Condition <- NA

tmpgg <- rbind(tmp3a, tmp3b)
tmpgg$Effect <- factor(tmpgg$Effect, levels=rev(study.order))
tmpgg$facet <- rep(c("Variate", "Effect"), c(nrow(tmp3a), nrow(tmp3b)))
tmpgg$facet <- factor(tmpgg$facet, levels=c("Variate", "Effect"))
tmp1 <- data.frame(Level="Lab", Effect="Quote Attribution", Condition="Condition 1",
		facet=c("Variate", "Variate", "Effect", "Effect"),
		value=c(0,1.75,	0,1.75))
tmp1$facet <- factor(tmp1$facet, levels=c("Variate", "Effect"))
plot3 <- ggplot(tmpgg, aes(x=Effect,y=sqrt(value), shape=Level, col=Condition)) + coord_flip() + 
	facet_grid(~facet, scales="free_x") +
	stat_summary(fun.y=median, geom="point", position=position_dodge(width=-2/3)) + 
	stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-2/3), show.legend=FALSE) +
	stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-2/3)) +
	xlab("Phenomenon") + ylab(expression(paste("Variation (", sigma[d], ")"))) + 
	theme(legend.position="bottom", legend.box="vertical") + 
	scale_shape_manual(values=c("Lab"=16, "Subject"=15)) + 
	scale_color_discrete(breaks=cond, na.value=1) +
	guides(shape = guide_legend(order = 1), color = guide_legend(order = 2)) +
	geom_point(data=tmp1, aes(y=value), col=NA)

ggsave("graph_paper_mlp3.pdf", plot=plot3, device=cairo_pdf, width=4/3*3.85, height=4/3*6.41)

tmp1 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- apply(aa4[,,grepl("lambda_beta1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,rep(1:nDepvar, each=nCond)]
tmp3 <- apply(aa4[,,grepl("lambda_beta2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp4 <- apply(aa4[,,grepl("lambda_beta3_d_raw\\[", dimnames(aa4)[[3]])], 3, identity)
tmp5 <- apply(aa4[,,grepl("lambda_beta3_d_raw", dimnames(aa4)[[3]])], 3, identity)
tmp6 <- apply(aa4[,,grepl("lambda_beta3_d_raw\\[", dimnames(aa4)[[3]])], 3, identity)
tmp6 <- tmp6[,rep(1:nAnch, each=nCond)]
tmp7 <- apply(aa4[,,grepl("lambda_beta4_cd_raw", dimnames(aa4)[[3]])], 3, identity)
tmpS <- apply(aa4[,,grepl("omega_phi_beta_f", dimnames(aa4)[[3]])], 3, identity)
for(i in 1:ncol(tmp1)){ tmp1[,i] <- 1 * tmpS[,1] }
for(i in 1:ncol(tmp2)){ tmp2[,i] <- tmp2[,i] * tmpS[,2] }
for(i in 1:ncol(tmp3)){ tmp3[,i] <- tmp3[,i] * tmpS[,3] }
for(i in 1:ncol(tmp4)){ tmp4[,i] <- tmpS[,4] }
for(i in 1:ncol(tmp5)){ tmp5[,i] <- tmpS[,5] }
for(i in 1:ncol(tmp6)){ tmp6[,i] <- tmp6[,i] * tmpS[,6] }
for(i in 1:ncol(tmp7)){ tmp7[,i] <- tmp7[,i] * tmpS[,7] }

tmpgg <- rbind( melt(tmp1),melt(tmp2),melt(tmp3),melt(tmp4),melt(tmp5),melt(tmp6),melt(tmp7) )
tmp.name <- c("Intercept", "Dependent Measure", "Variate",
"Anchoring Intercepts", "Anchoring Intercepts", "Anchoring Dependent Measure", "Anchoring Variate")
tmpgg$Factor <- rep(tmp.name, c(length(tmp1),length(tmp2),length(tmp3),
				length(tmp4),length(tmp5),length(tmp6),length(tmp7)) )
tmpgg$Factor <- factor(tmpgg$Factor, levels=unique(tmp.name))
tmpgg$Effect <- c( rep(study,each=nCond*nrow(tmp1)), rep(study,each=nCond*nrow(tmp2)), rep(study,each=nCond*nrow(tmp3)),
					rep(study[1:nAnch],each=nrow(tmp4)), rep(study[1:nAnch],each=nrow(tmp5)),
					rep(study[1:nAnch],each=nCond*nrow(tmp6)), rep(study[1:nAnch],each=nCond*nrow(tmp7)) )
tmpgg$Effect <- factor(tmpgg$Effect, levels=rev(study.order))
tmpc <- rep(paste("Condition", 1:nCond), each=nrow(tmp3))
tmpgg$Condition <- c(rep(tmpc,nDepvar), rep(tmpc,nDepvar), rep(tmpc,nDepvar),
						rep("Condition 1",length(tmp4)), rep("Condition 2",length(tmp5)),
						rep(tmpc,nAnch), rep(tmpc,nAnch) )
tmp1 <- data.frame(Factor=factor(unique(tmp.name),levels=unique(tmp.name)), yy=c(NA,0,0,NA,0,0))
tmp2 <- data.frame(Factor=factor(rep(unique(tmp.name),each=2), levels=unique(tmp.name)),
						Effect=tmpgg$Effect[1], Condition=tmpgg$Condition[1],
						value=c(0.025,0.15,   -.625,.625,      -.425,.425,
								0.01,0.18,   -.11,.11,  -.525,.525))
plot1 <- ggplot(tmpgg, aes(x=Effect,y=value,col=Condition)) + coord_flip() + 
		facet_grid(~Factor, scales="free_x") +
		stat_summary(fun.y=median, geom="point", position=position_dodge(width=-1/3)) + 
		stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-1/3), show.legend=FALSE) +
		stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-1/3)) +
		xlab("Phenomenon") + ylab(expression(paste("Loading (", sigma[d], ")"))) + 
		geom_hline(data=tmp1,aes(yintercept=yy),linetype=2,size=0.25) +
		geom_point(data=tmp2,col=NA) +
		theme(legend.position="bottom")

ggsave("graph_paper_mlp1.pdf", plot=plot1, device=cairo_pdf, width=4/3*9, height=4/3*6)

tmp1 <- apply(aa4[,,grepl("lambda_gamma2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- apply(aa4[,,grepl("lambda_gamma1_d", dimnames(aa4)[[3]])], 3, identity)
tmp2 <- tmp2[,rep(1:nDepvar, each=nCond)]
tmp3 <- apply(aa4[,,grepl("lambda_gamma2_cd", dimnames(aa4)[[3]])], 3, identity)
tmp4 <- apply(aa4[,,grepl("lambda_gamma3_cd_raw\\[", dimnames(aa4)[[3]])], 3, identity)
tmp5 <- apply(aa4[,,grepl("lambda_gamma3_cd_raw", dimnames(aa4)[[3]])], 3, identity)
tmpS <- apply(aa4[,,grepl("omega_phi_gamma_f", dimnames(aa4)[[3]])], 3, identity)
for(i in 1:ncol(tmp1)){ tmp1[,i] <- 1 * tmpS[,1] }
for(i in 1:ncol(tmp2)){ tmp2[,i] <- tmp2[,i] * tmpS[,2] }
for(i in 1:ncol(tmp3)){ tmp3[,i] <- tmp3[,i] * tmpS[,3] }
for(i in 1:ncol(tmp4)){ tmp4[,i] <- 1 * tmpS[,4] }
for(i in 1:ncol(tmp5)){ tmp5[,i] <- tmp5[,i] * tmpS[,5] }

tmpgg <- rbind( melt(tmp1),melt(tmp2),melt(tmp3),melt(tmp4),melt(tmp5) )
tmp.name <- c("Intercept", "Dependent Measure", "Variate", "Anchoring Intercept", "Anchoring Variate")
tmpgg$Factor <- rep(tmp.name, c(length(tmp1),length(tmp2),length(tmp3),
				length(tmp4),length(tmp5)) )
tmpgg$Factor <- factor(tmpgg$Factor, levels=unique(tmp.name))
tmpgg$Effect <- c( rep(study,each=nCond*nrow(tmp1)), rep(study,each=nCond*nrow(tmp2)), rep(study,each=nCond*nrow(tmp3)),
					rep(study[1:nAnch],each=nCond*nrow(tmp4)), rep(study[1:nAnch],each=nCond*nrow(tmp5)) )
tmpgg$Effect <- factor(tmpgg$Effect, levels=rev(study.order))
tmpc <- rep(paste("Condition", 1:nCond), each=nrow(tmp3))
tmpgg$Condition <- c(rep(tmpc,nDepvar), rep(tmpc,nDepvar), rep(tmpc,nDepvar),
						rep(tmpc,nAnch), rep(tmpc,nAnch) )
tmp1 <- data.frame(Factor=factor(unique(tmp.name),levels=unique(tmp.name)), yy=c(NA,0,0,NA,0))
tmp2 <- data.frame(Factor=factor(rep(unique(tmp.name),each=2), levels=unique(tmp.name)),
						Effect=tmpgg$Effect[1], Condition=tmpgg$Condition[1],
						value=c(0.20,0.28,   -1.2,1.2,      -.7,.7,
								0.25,0.39,  -1.1,1.1))
plot2 <- ggplot(tmpgg, aes(x=Effect,y=value,col=Condition)) + coord_flip() + 
		facet_grid(~Factor, scales="free_x") +
		stat_summary(fun.y=median, geom="point", position=position_dodge(width=-1/3)) + 
		stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-1/3), show.legend=FALSE) +
		stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-1/3)) +
		xlab("Phenomenon") + ylab(expression(paste("Loading (", sigma[d], ")"))) + 
		geom_hline(data=tmp1,aes(yintercept=yy),linetype=2,size=0.25) +
		geom_point(data=tmp2,col=NA) +
		theme(legend.position="bottom")

ggsave("graph_paper_mlp2.pdf", plot=plot2, device=cairo_pdf, width=4/3*7.7, height=4/3*6)

ulab <- sort(unique(dat$Lab))
ulab.name <- c("Penn State (Abington), United States",
"University of Brasília, Brazil",
"Charles University, Czech Republic",
"Connecticut College, United States",
"California State University, United States",
"HELP University, Malaysia",
"Ithaca College, United States",
"James Madison University, United States",
"Koç University, Turkey",
"Wilfrid Laurier University, Canada",
"London School of Economics, United Kingdom",
"Loyola University Chicago, United States",
"McDaniel College, United States",
"Mount Saint Vincent University, Canada",
"Amazon Mechanical Turk, United States",
"Ohio State University, United States",
"Occidental College, United States",
"Project Implicit, United States",
"Penn State (State College), United States",
"Queens College (Lab 1), United States",
"Queens College (Lab 2), United States",
"San Diego State University, United States",
"SWPS University (Sopot), Poland",
"badania.net, Poland",
"Texas A&M (College Station), United States",
"Texas A&M (Commerce), United States",
"Texas A&M (College Station Online), United States",
"Tilburg University, Netherlands",
"University of Florida, United States",
"University of Padua, Italy",
"University of Virginia, United States",
"Virginia Commonwealth University, United States",
"University of Wisconsin-Madison, United States",
"Western Kentucky University, United States",
"Washington and Lee University, United States",
"Worcester Polytechnic Institute, United States")
names(ulab.name) <- as.character(ulab)

tmp <- apply(aa4[,,grepl("beta_lcd", dimnames(aa4)[[3]])], 3, identity)
tmpd <- array(NA, c(nrow(tmp), nLab, 2))
for(i in 1:nrow(tmp)){
	tmpb <- matrix(tmp[i,], nrow=nLab)
	tmpd[i,,1] <- apply(tmpb, 1, function(x){sqrt(mean(x^2))})
	tmpc <- t(apply(tmpb, 1, function(x){cc%*%x}))
	tmpd[i,,2] <- apply(tmpc, 1, function(x){sqrt(mean(x^2))})
}
tmpgg <- melt(tmpd)
tmpgg$Lab <- ulab[tmpgg$Var2]
tmpgg$facet <- c("Variate", "Effect")[tmpgg$Var3]
tmpgg$facet <- factor(tmpgg$facet, levels=c("Variate", "Effect"))
tmp1 <- data.frame(LabName=ulab.name[1],
	facet=c("Variate", "Variate", "Effect", "Effect"),
	value=c(0,0.9,	0,0.9))
tmp1$facet <- factor(tmp1$facet, levels=c("Variate", "Effect"))

tmpgg$LabName <- ulab.name[tmpgg$Lab]
tmpo <- names(sort(by(tmpgg$value[tmpgg$facet=="Variate"], tmpgg$LabName[tmpgg$facet=="Variate"], median)))
tmpgg$LabName <- factor(tmpgg$LabName, levels=rev(tmpo))

plot5 <- ggplot(tmpgg, aes(x=LabName,y=value)) + coord_flip() + facet_wrap(~facet, scales="free_x") +
	stat_summary(fun.y=median, geom="point", position=position_dodge(width=-2/3)) + 
	stat_summary(fun.data=f50, geom="errorbar", width=0, size=1,  position=position_dodge(width=-2/3), show.legend=FALSE) +
	stat_summary(fun.data=f95, geom="errorbar", width=0,  position=position_dodge(width=-2/3)) +
	xlab("Lab") + ylab(expression(paste("Root Mean Square Error (", sigma[d], ")"))) +
	geom_point(data=tmp1, aes(y=value), col=NA)

ggsave("graph_paper_mlp5.pdf", plot=plot5, device=cairo_pdf, width=4/3*4.89, height=4/3*6.6)

init.fcn <- function(x){
	nFac_bg <- 7
	nFac_d <- 5
	out <- list(
		alpha_cd_rawCB = matrix(x[grepl("alpha_cd_rawCB",names(x))],nCond,nDepvarC+nDepvarB),
		alpha_cd_rawO = matrix(x[grepl("alpha_cd_rawO",names(x))],nCond-1,nDepvarO),
		sigma_dC = x[grepl("sigma_dC",names(x))],
		cutpoints_dO = matrix(x[grepl("cutpoints_dO",names(x))],nDepvarO,nCatO-1),
	
		phi_beta_lf  = matrix(x[grepl("phi_beta_lf",names(x))],nLab,nFac_bg),
		omega_phi_beta_f = x[grepl("omega_phi_beta_f",names(x))],
		Rho_phi_beta_ff  = matrix(x[grepl("Rho_phi_beta_ff",names(x))],nFac_bg,nFac_bg),
		lambda_beta1_d = x[grepl("lambda_beta1_d",names(x))],
		lambda_beta2_cd  = matrix(x[grepl("lambda_beta2_cd",names(x))],nCond,nDepvar),
		lambda_beta3_d_raw = x[grepl("lambda_beta3_d_raw",names(x))],
		lambda_beta4_cd_raw  = matrix(x[grepl("lambda_beta4_cd_raw",names(x))],nCond,nAnch),
	
		eta_ldc = array(x[grepl("eta_ldc",names(x))], c(nLab,nDepvar,nCond)),
		omega_eta_dc  = matrix(x[grepl("omega_eta_dc",names(x))],nDepvar,nCond),
		Rho_eta_dcc = array(x[grepl("Rho_eta_dcc",names(x))], c(nDepvar,nCond,nCond)),
	
		phi_gamma_sf  = matrix(x[grepl("phi_gamma_sf",names(x))],nSubj,nFac_d),
		omega_phi_gamma_f = x[grepl("omega_phi_gamma_f",names(x))],
		Rho_phi_gamma_ff  = matrix(x[grepl("Rho_phi_gamma_ff",names(x))],nFac_d,nFac_d),
		lambda_gamma1_d = x[grepl("lambda_gamma1_d",names(x))],
		lambda_gamma2_cd  = matrix(x[grepl("lambda_gamma2_cd",names(x))],nCond,nDepvar),
		lambda_gamma3_cd_raw  = matrix(x[grepl("lambda_gamma3_cd_raw",names(x))],nCond,nAnch)
	)
	return(out)
}

starting.value <- init.fcn(apply(aa[,1,], 2, median))
starting.value.processed <- init.fcn(apply(aa4[,1,], 2, median))
starting.value.relabel <- starting.value

tmp <- table(sign(starting.value$lambda_beta1_d), sign(starting.value.processed$lambda_beta1_d))
flip1 <- ifelse(sum(diag(tmp)) / sum(tmp) > 0.5, FALSE, TRUE)
tmp <- table(sign(starting.value$lambda_beta2_cd), sign(starting.value.processed$lambda_beta2_cd))
flip2 <- ifelse(sum(diag(tmp)) / sum(tmp) > 0.5, FALSE, TRUE)
tmp <- table(sign(starting.value$lambda_beta3_d_raw), sign(starting.value.processed$lambda_beta3_d_raw))
flip3 <- ifelse(sum(diag(tmp)) / sum(tmp) > 0.5, FALSE, TRUE)
tmp <- table(sign(starting.value$lambda_beta4_cd_raw), sign(starting.value.processed$lambda_beta4_cd_raw))
flip4 <- ifelse(sum(diag(tmp)) / sum(tmp) > 0.5, FALSE, TRUE)

if(flip1 | flip2 | flip3 | flip4){
	tmpl2 <- starting.value$lambda_beta1_d
	tmpl3 <- starting.value$lambda_beta2_cd
	tmpl6 <- starting.value$lambda_beta3_d_raw
	tmpl7 <- starting.value$lambda_beta4_cd_raw
	tmpp <- starting.value$phi_beta_lf
	tmpo <- starting.value$omega_phi_beta_f
	tmpR <- starting.value$Rho_phi_beta_ff

	LL <- cbind(1, rep(tmpl2,each=nCond), as.vector(tmpl3), 
				rep(Anch_ind,each=nCond) * rep(1:0, nDepvar),
				rep(Anch_ind,each=nCond) * rep(0:1, nDepvar),
				c(rep(tmpl6,each=nCond), rep(0,nCond*(nDepvar-nAnch))),
				c(as.vector(tmpl7), rep(0,nCond*(nDepvar-nAnch))))
	OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
	VV <- LL %*% OO %*% t(LL)
	
	PP <- diag(7)
	if(flip1){ PP[2,2] <- -1 }
	if(flip2){ PP[3,3] <- -1 }
	if(flip3){ PP[6,6] <- -1 }
	if(flip4){ PP[7,7] <- -1 }

	LLr <- LL %*% PP
	OOr <- solve(PP) %*% OO %*% t(solve(PP))
	RRr <- cov2cor(OOr)
	oor <- sqrt(diag(OOr))
	ppr <- t(solve(PP) %*% t(tmpp))

	starting.value.relabel$lambda_beta1_d <- LLr[seq(1,nCond*nDepvar,by=nCond), 2]
	starting.value.relabel$lambda_beta2_cd <- matrix(LLr[,3], nCond, nDepvar)
	starting.value.relabel$lambda_beta3_d_raw <- LLr[seq(1,nCond*nAnch,by=nCond), 6]
	starting.value.relabel$lambda_beta4_cd_raw <- matrix(LLr[1:(nCond*nAnch),7], nCond, nAnch)
	starting.value.relabel$phi_beta_lf <- ppr
	starting.value.relabel$omega_phi_beta_f <- oor
	starting.value.relabel$Rho_phi_beta_ff <- RRr
}

tmp <- table(sign(starting.value$lambda_gamma1_d), sign(starting.value.processed$lambda_gamma1_d))
flip1 <- ifelse(sum(diag(tmp)) / sum(tmp) > 0.5, FALSE, TRUE)
tmp <- table(sign(starting.value$lambda_gamma2_cd), sign(starting.value.processed$lambda_gamma2_cd))
flip2 <- ifelse(sum(diag(tmp)) / sum(tmp) > 0.5, FALSE, TRUE)
tmp <- table(sign(starting.value$lambda_gamma3_cd_raw), sign(starting.value.processed$lambda_gamma3_cd_raw))
flip3 <- ifelse(sum(diag(tmp)) / sum(tmp) > 0.5, FALSE, TRUE)

if(flip1 | flip2 | flip3){
	tmpl2 <- starting.value$lambda_gamma1_d
	tmpl3 <- starting.value$lambda_gamma2_cd
	tmpl5 <- starting.value$lambda_gamma3_cd_raw
	tmpp <- starting.value$phi_gamma_sf
	tmpo <- starting.value$omega_phi_gamma_f
	tmpR <- starting.value$Rho_phi_gamma_ff

	LL <- cbind(1, rep(tmpl2,each=nCond), as.vector(tmpl3), rep(Anch_ind,each=nCond),
				c(as.vector(tmpl5), rep(0,nCond*(nDepvar-nAnch))))
	OO <- diag(tmpo) %*% tmpR %*% diag(tmpo)
	VV <- LL %*% OO %*% t(LL)
	
	PP <- diag(5)
	if(flip1){ PP[2,2] <- -1 }
	if(flip2){ PP[3,3] <- -1 }
	if(flip3){ PP[5,5] <- -1 }

	LLr <- LL %*% PP
	OOr <- solve(PP) %*% OO %*% t(solve(PP))
	RRr <- cov2cor(OOr)
	oor <- sqrt(diag(OOr))
	ppr <- t(solve(PP) %*% t(tmpp))

	starting.value.relabel$lambda_gamma1_d <- LLr[seq(1,nCond*nDepvar,by=nCond), 2]
	starting.value.relabel$lambda_gamma2_cd <- matrix(LLr[,3], nCond, nDepvar)
	starting.value.relabel$lambda_gamma3_cd_raw <- matrix(LLr[1:(nCond*nAnch),5], nCond, nAnch)
	starting.value.relabel$phi_gamma_lf <- ppr
	starting.value.relabel$omega_phi_gamma_f <- oor
	starting.value.relabel$Rho_phi_gamma_ff <- RRr
}

saveRDS(starting.value.relabel, file="posterior1.init.rds")
