# Load packages and read in data
library(ggplot2)
library(reshape2)
library(abind)
library(rstan)
options(mc.cores = parallel::detectCores())

processed.data <- readRDS("manylabs.data.rds")
YY <- processed.data$YY
TT <- processed.data$TT
lab <- processed.data$lab.id
study <- processed.data$study.info[,1]
var.type <- processed.data$study.info[,2]
XX <- data.frame(x1 = 4-as.numeric(processed.data$political.ideology))

# Prepare data for Stan
dat <- melt(YY)
colnames(dat)[1:2] <- c("SubjectID","EffectID")
dat$Treatment <- melt(TT)$value + 1
dat$Lab <- lab
dat$LabID <- as.numeric(dat$Lab)
dat$VarType <- var.type[dat$EffectID]

sel <- rowSums(is.na(dat)) > 0 | rowSums(is.na(XX)) > 0
dat <- dat[!sel,]
XX[is.na(XX)] <- -999

# Scalars for stan
nCond <- 2		# All DVs have two conditions
nCatO <- 9		# Both ordinal DVs are 1:9
nCov <- ncol(XX)
nLab <- max(dat$LabID)
nSubj <- max(dat$SubjectID)
nRowC <- sum(dat$VarType=="C")
nRowB <- sum(dat$VarType=="B")
nRowO <- sum(dat$VarType=="O")
nDepvar <- length(unique(dat$EffectID))
nDepvarC <- length(unique(dat$EffectID[dat$VarType=="C"]))
nDepvarB <- length(unique(dat$EffectID[dat$VarType=="B"]))
nDepvarO <- length(unique(dat$EffectID[dat$VarType=="O"]))

# Continuous for stan
datC <- dat[dat$VarType=="C",]
datC$EffectID2 <- as.numeric(as.factor(datC$EffectID))
s.pool <- rep(NA, max(datC$EffectID2))
for(i in 1:length(s.pool)){
	sel <- datC$EffectID2==i
	tmpv <- by(datC$value[sel], list(datC$Lab[sel], datC$Treatment[sel]), var)
	tmpn <- by(datC$value[sel], list(datC$Lab[sel], datC$Treatment[sel]), length)
	s.pool[i] <- sqrt( sum((tmpn-1)*tmpv, na.rm=TRUE) / sum(tmpn-1, na.rm=TRUE) )
}

# Binary for stan
datB <- dat[dat$VarType=="B",]
datB$EffectID2 <- as.numeric(as.factor(datB$EffectID)) + nDepvarC

# Ordinal for stan
datO <- dat[dat$VarType=="O",]
datO$EffectID2 <- as.numeric(as.factor(datO$EffectID)) + nDepvarC + nDepvarB

study.orig <- study
var.type.orig <- var.type
study <- study[c(unique(datC$EffectID), unique(datB$EffectID), unique(datO$EffectID))]
var.type <- var.type[c(unique(datC$EffectID), unique(datB$EffectID), unique(datO$EffectID))]
Anch_ind <- 1*grepl("Anchoring", study)
nAnch <- sum(Anch_ind)

# Compile model and run sampler
starting.value <- readRDS("posterior1.init.rds")
names(starting.value)[[1]] <- "alpha_int_cd_rawCB"
names(starting.value)[[2]] <- "alpha_int_cd_rawO"
starting.value$alpha_XX_cd <- array(0, c(nCond,nDepvar,nCov))
init.fcn <- function(){return(starting.value)}
stan.model <- stan_model(file="model2.stan")
post <- sampling(stan.model, seed=2468, verbose=TRUE, refresh=10,
				  pars=c("muC","sigC","muB","muO","log_likO"), include=FALSE,
				  data=list(
					  nCond=nCond, nCatO=nCatO, nCov=nCov, nLab=nLab, nSubj=nSubj,
					  nRowC=nRowC, nRowB=nRowB, nRowO=nRowO, 
					  nDepvar=nDepvar, nDepvarC=nDepvarC, nDepvarB=nDepvarB, nDepvarO=nDepvarO,
					  yC=datC$value, subjC=datC$SubjectID, labC=datC$LabID,
					  depvarC=datC$EffectID2, conditionC=datC$Treatment,  dv_scale=s.pool,
					  yB=datB$value, subjB=datB$SubjectID, labB=datB$LabID,
					  depvarB=datB$EffectID2, conditionB=datB$Treatment,
					  yO=datO$value, subjO=datO$SubjectID, labO=datO$LabID,
					  depvarO=datO$EffectID2, conditionO=datO$Treatment,
					  Anch_ind=Anch_ind, XX=XX
				  ), 
				  init=init.fcn
				 )

# Post-process chains not necessary for the alphas

# Plot output
study.orig <- study
study <- c(
	"Anchoring:\nDistance from SF to NYC",
	"Anchoring:\nPopulation of Chicago",
	"Anchoring:\nHeight of Mount Everest",
	"Anchoring:\nUS Babies Born Per Day",
	"Retrospective\nGambler Fallacy",
	"Flag Priming",
	"Currency Priming",
	"Imagined Contact",
	"Sex Differences in\nImplicit Math Attitudes",
	"Gain vs. Loss Framing",
	"Low vs. High\nCategory Scales",
	"Norm of Reciprocity",
	"Allowed vs. Forbidden",
	"Sunk Costs",
	"Quote Attribution")
study.order <- c(					# Plot in same order as model1
	"Gain vs. Loss Framing",
	"Retrospective\nGambler Fallacy", 
	"Sex Differences in\nImplicit Math Attitudes",
	"Sunk Costs",
	"Quote Attribution", 
	"Low vs. High\nCategory Scales",
	"Allowed vs. Forbidden",
	"Currency Priming",
	"Imagined Contact",
	"Norm of Reciprocity",
	"Flag Priming",
	"Anchoring:\nDistance from SF to NYC", 
	"Anchoring:\nUS Babies Born Per Day",
	"Anchoring:\nPopulation of Chicago", 
	"Anchoring:\nHeight of Mount Everest")

fun.alpha <- function(y,alpha=0.05){ 
	out <- quantile(y,probs=c(alpha/2,0.5,1-alpha/2))
	names(out) <- c("ymin","y","ymax")
	out
}
f95 <- function(y){ fun.alpha(y,0.05) }
f50 <- function(y){ fun.alpha(y,0.50) }
ppp <- c(0.025, 0.25,   0.50,   0.75, 0.975)
qqq <- qnorm(ppp)

cc <- matrix(0, nDepvar, nCond*nDepvar)
for(i in 1:nDepvar){
	tmpj <- (i-1)*nCond + 1
	cc[i,tmpj:(tmpj+nCond-1)] <- c(-1,1)
}
modk <- function(x,k){ tmp <- x%%k; ifelse(tmp==0,k,tmp) }
cond <- c("Condition 1", "Condition 2")

aa <- as.array(post)[,,grepl("alpha", names(post)) | grepl("cutpoints", names(post))]
xx <- seq(-3, 3, length=101)
idxO <- seq(1, nDepvarO*(nCatO-1), by=nDepvarO)
tmpi <- apply(aa[,,grepl("alpha_int_cd\\[", dimnames(aa)[[3]])], 3, identity)
tmpi.cut <- apply(aa[,,grepl("cutpoints_dO", dimnames(aa)[[3]])], 3, identity)
tmpi[,27] <- rowSums(tmpi.cut[,idxO])/nCatO - tmpi[,28]/2
tmpi[,28] <- rowSums(tmpi.cut[,idxO])/nCatO + tmpi[,28]/2
tmpi[,29] <- rowSums(tmpi.cut[,idxO+1])/nCatO - tmpi[,30]/2
tmpi[,30] <- rowSums(tmpi.cut[,idxO+1])/nCatO + tmpi[,30]/2
tmps <- apply(aa[,,grepl("alpha_XX_cd", dimnames(aa)[[3]])], 3, identity)
tmpxa <- array(NA, c(dim(tmpi), length(xx)))
for(i in 1:length(xx)){
	tmpxa[,,i] <- tmpi + tmps*xx[i]	
}
for(i in seq(1,nCond*nDepvar,by=nCond)){
	tmpxa[,i:(i+nCond-1),] <- tmpxa[,i:(i+nCond-1),] - mean( tmpxa[,i:(i+nCond-1),] )
}
tmp3 <- melt(apply(tmpxa, 2:3, median))
tmp3$Effect <- study[ceiling(tmp3$Var1/2)]
tmp3$Effect <- factor(tmp3$Effect, levels=study.order)
tmp3$Condition <- paste("Condition", -(tmp3$Var1%%2) + 2)
tmp3$xx <- xx[tmp3$Var2] + 4
tmp3$ym <- melt(apply(tmpxa, 2:3, quantile, probs=0.025))$value
tmp3$yM <- melt(apply(tmpxa, 2:3, quantile, probs=0.975))$value
tmp4 <- apply(cbind(by(tmp3$ym, tmp3$Effect, min), by(tmp3$yM, tmp3$Effect, max)), 
					1, function(x){max(abs(x))})
tmp4 <- data.frame(Effect=rep(names(tmp4),2), xx=mean(xx)+4, value=c(tmp4,-tmp4))
tmp4$Effect <- factor(tmp4$Effect, levels=study.order)
plot4 <- ggplot(tmp3, aes(x=xx,y=value,col=Condition)) + facet_wrap(~Effect, scales="free_y") +  
	 geom_line() + 
	 geom_line(aes(y=ym),linetype=2,size=0.25) +
	 geom_line(aes(y=yM),linetype=2,size=0.25) +
	 scale_x_continuous("Political Ideology", breaks=c(1,3,5,7)) +
	 ylab(expression(paste("Value (", sigma[d], ")"))) + theme(legend.position="bottom") +
	 geom_point(data=tmp4,col=NA)

ggsave("graph_paper_mlp4.pdf", plot=plot4, device=cairo_pdf, width=4/3*6.5, height=4/3*7.25)
